@@ -1,7 +1,7 @@
 /*
  * file: hot/hot.c
  *
- * $Id: hot.c,v 1.16 2000/05/22 17:10:00 mitch Exp $
+ * $Id: hot.c,v 1.38 2004/10/12 21:48:38 mitch Exp $
  */
 
 /*
@@ -40,7 +40,7 @@
  * CHROMA_LIM is the limit (in IRE units) of the overall
  *	chrominance amplitude; it should be 50 or perhaps
  *	very slightly higher.
- * 
+ *
  * COMPOS_LIM is the maximum amplitude (in IRE units) allowed for
  *	the composite signal.  A value of 100 is the maximum
  *	monochrome white, and is always safe.  120 is the absolute
@@ -77,42 +77,26 @@
 
 #include "libgimp/stdplugins-intl.h"
 
-
-struct Grgb
-{
-  guint8 red;
-  guint8 green;
-  guint8 blue;
-};
-
-struct GRegion
-{
-  gint32 x;
-  gint32 y;
-  gint32 width;
-  gint32 height;
-};
-
-struct piArgs
+typedef struct
 {
   gint32 image;
   gint32 drawable;
   gint32 mode;
   gint32 action;
   gint32 new_layerp;
-};
+} piArgs;
 
 typedef enum
 {
-  act_lredux = 0,
-  act_sredux = 1,
-  act_flag   = 2
+  ACT_LREDUX,
+  ACT_SREDUX,
+  ACT_FLAG
 } hotAction;
 
 typedef enum
 {
-  mode_ntsc = 0,
-  mode_pal  = 1
+  MODE_NTSC,
+  MODE_PAL
 } hotModes;
 
 #define	CHROMA_LIM      50.0		/* chroma amplitude limit */
@@ -159,18 +143,18 @@ static glong	ichroma_lim2;        /* chroma limit squared (scaled integer) */
 static gint	icompos_lim;         /* composite amplitude limit (scaled integer) */
 
 static void query        (void);
-static void run          (gchar   *name,
-			  gint     nparam,
-			  GParam  *param,
-			  gint    *nretvals,
-			  GParam **retvals);
+static void run          (const gchar      *name,
+			  gint              nparam,
+			  const GimpParam  *param,
+			  gint             *nretvals,
+			  GimpParam       **retvals);
 
-static gint pluginCore   (struct piArgs *argp);
-static gint pluginCoreIA (struct piArgs *argp);
-static gint hotp         (register guint8 r,
-			  register guint8 g,
-			  register guint8 b);
-static void build_tab    (int m);
+static gint pluginCore   (piArgs   *argp);
+static gint pluginCoreIA (piArgs   *argp);
+static gboolean hotp     (guint8  r,
+			  guint8  g,
+			  guint8  b);
+static void build_tab    (gint    m);
 
 /*
  * gc: apply the gamma correction specified for this video standard.
@@ -196,7 +180,7 @@ static void build_tab    (int m);
 #define pix_decode(v)  ((double)v / (double)MAXPIX)
 #define pix_encode(v)  ((int)(v * (double)MAXPIX + 0.5))
 
-GPlugInInfo PLUG_IN_INFO =
+GimpPlugInInfo PLUG_IN_INFO =
 {
   NULL,  /* init_proc  */
   NULL,  /* quit_proc  */
@@ -209,16 +193,15 @@ MAIN ()
 static void
 query (void)
 {
-  static GParamDef args[] =
+  static GimpParamDef args[] =
   {
-    { PARAM_INT32, "run_mode", "Interactive, non-interactive" },
-    { PARAM_IMAGE, "image", "The Image" },
-    { PARAM_DRAWABLE, "drawable", "The Drawable" },
-    { PARAM_INT32, "mode", "Mode -- NTSC/PAL" },
-    { PARAM_INT32, "action", "The action to perform" },
-    { PARAM_INT32, "new_layerp", "Create a new layer iff True" }
+    { GIMP_PDB_INT32, "run_mode", "Interactive, non-interactive" },
+    { GIMP_PDB_IMAGE, "image", "The Image" },
+    { GIMP_PDB_DRAWABLE, "drawable", "The Drawable" },
+    { GIMP_PDB_INT32, "mode", "Mode -- NTSC/PAL" },
+    { GIMP_PDB_INT32, "action", "The action to perform" },
+    { GIMP_PDB_INT32, "new_layerp", "Create a new layer iff True" }
   };
-  static gint nargs = sizeof (args) / sizeof (args[0]);
 
   gimp_install_procedure ("plug_in_hot",
 			  "Look for hot NTSC or PAL pixels ",
@@ -231,63 +214,65 @@ query (void)
 			  "Eric L. Hernes, Alan Wm Paeth",
 			  "Eric L. Hernes",
 			  "1997",
-			  N_("<Image>/Filters/Colors/Hot..."),
+			  N_("_Hot..."),
 			  "RGB",
-			  PROC_PLUG_IN,
-			  nargs, 0,
+			  GIMP_PLUGIN,
+			  G_N_ELEMENTS (args), 0,
 			  args, NULL);
+
+  gimp_plugin_menu_register ("plug_in_hot", "<Image>/Filters/Colors");
 }
 
 static void
-run (gchar   *name,
-     gint     nparam,
-     GParam  *param,
-     gint    *nretvals,
-     GParam **retvals)
+run (const gchar      *name,
+     gint              nparam,
+     const GimpParam  *param,
+     gint             *nretvals,
+     GimpParam       **retvals)
 {
-  static GParam rvals[1];
-
-  struct piArgs args;
+  static GimpParam rvals[1];
+  piArgs    args;
 
   *nretvals = 1;
-  *retvals = rvals;
+  *retvals  = rvals;
 
-  memset (&args, (int) 0, sizeof (struct piArgs));
+  INIT_I18N ();
+
+  memset (&args, (int) 0, sizeof (args));
   args.mode = -1;
 
   gimp_get_data ("plug_in_hot", &args);
 
-  args.image = param[1].data.d_image;
+  args.image    = param[1].data.d_image;
   args.drawable = param[2].data.d_drawable;
 
-  rvals[0].type = PARAM_STATUS;
-  rvals[0].data.d_status = STATUS_SUCCESS;
+  rvals[0].type          = GIMP_PDB_STATUS;
+  rvals[0].data.d_status = GIMP_PDB_SUCCESS;
+
   switch (param[0].data.d_int32)
     {
-    case RUN_INTERACTIVE:
-      INIT_I18N_UI();
+    case GIMP_RUN_INTERACTIVE:
       /* XXX: add code here for interactive running */
       if (args.mode == -1)
 	{
-	  args.mode       = mode_ntsc;
-	  args.action     = act_lredux;
+	  args.mode       = MODE_NTSC;
+	  args.action     = ACT_LREDUX;
 	  args.new_layerp = 1;
 	}
 
       if (pluginCoreIA(&args) == -1)
 	{
-	  rvals[0].data.d_status = STATUS_EXECUTION_ERROR;
+	  rvals[0].data.d_status = GIMP_PDB_EXECUTION_ERROR;
 	}
-      gimp_set_data ("plug_in_hot", &args, sizeof (struct piArgs));
+      gimp_set_data ("plug_in_hot", &args, sizeof (args));
 
     break;
 
-    case RUN_NONINTERACTIVE:
-      INIT_I18N();
+    case GIMP_RUN_NONINTERACTIVE:
       /* XXX: add code here for non-interactive running */
       if (nparam != 6)
 	{
-	  rvals[0].data.d_status = STATUS_CALLING_ERROR;
+	  rvals[0].data.d_status = GIMP_PDB_CALLING_ERROR;
 	  break;
 	}
       args.mode       = param[3].data.d_int32;
@@ -296,32 +281,32 @@ run (gchar   *name,
 
       if (pluginCore(&args) == -1)
 	{
-	  rvals[0].data.d_status = STATUS_EXECUTION_ERROR;
+	  rvals[0].data.d_status = GIMP_PDB_EXECUTION_ERROR;
 	  break;
 	}
     break;
 
-    case RUN_WITH_LAST_VALS:
-      INIT_I18N();
+    case GIMP_RUN_WITH_LAST_VALS:
       /* XXX: add code here for last-values running */
       if (pluginCore (&args) == -1)
 	{
-	  rvals[0].data.d_status = STATUS_EXECUTION_ERROR;
+	  rvals[0].data.d_status = GIMP_PDB_EXECUTION_ERROR;
 	}
     break;
   }
 }
 
 static gint
-pluginCore (struct piArgs *argp)
+pluginCore (piArgs *argp)
 {
-  GDrawable *drw, *ndrw=NULL;
-  GPixelRgn srcPr, dstPr;
+  GimpDrawable *drw, *ndrw=NULL;
+  GimpPixelRgn srcPr, dstPr;
   gint retval = 0;
   gint nl=0;
   gint y, x, i;
   gint Y, I, Q;
-  guint width, height, Bpp;
+  guint width, height, bpp;
+  gint sel_x1, sel_x2, sel_y1, sel_y2;
   gint prog_interval;
   guchar *src, *s, *dst, *d;
   guchar r, prev_r=0, new_r=0;
@@ -330,12 +315,12 @@ pluginCore (struct piArgs *argp)
   gdouble fy, fc, t, scale;
   gdouble pr, pg, pb;
   gdouble py;
-  
+
   drw = gimp_drawable_get (argp->drawable);
 
   width = drw->width;
   height = drw->height;
-  Bpp = drw->bpp;
+  bpp = drw->bpp;
   if (argp->new_layerp)
     {
       gchar name[40];
@@ -356,48 +341,59 @@ pluginCore (struct piArgs *argp)
 		  action_names[argp->action]);
 
       nl = gimp_layer_new (argp->image, name, width, height,
-			   RGBA_IMAGE, (gdouble)100, NORMAL_MODE);
+			   GIMP_RGBA_IMAGE, (gdouble)100, GIMP_NORMAL_MODE);
       ndrw = gimp_drawable_get (nl);
-      gimp_drawable_fill (nl, TRANS_IMAGE_FILL);
+      gimp_drawable_fill (nl, GIMP_TRANSPARENT_FILL);
       gimp_image_add_layer (argp->image, nl, 0);
     }
 
-  src = g_new (guchar, width * height * Bpp);
+  gimp_drawable_mask_bounds (drw->drawable_id,
+			     &sel_x1, &sel_y1, &sel_x2, &sel_y2);
+
+  width = sel_x2 - sel_x1;
+  height = sel_y2 - sel_y1;
+
+  src = g_new (guchar, width * height * bpp);
   dst = g_new (guchar, width * height * 4);
-  gimp_pixel_rgn_init (&srcPr, drw, 0, 0, width, height, FALSE, FALSE);
+  gimp_pixel_rgn_init (&srcPr, drw, sel_x1, sel_y1, width, height,
+                       FALSE, FALSE);
+
   if (argp->new_layerp)
     {
-      gimp_pixel_rgn_init (&dstPr, ndrw, 0, 0, width, height, FALSE, FALSE);
+      gimp_pixel_rgn_init (&dstPr, ndrw, sel_x1, sel_y1, width, height,
+                           FALSE, FALSE);
     }
   else
     {
-      gimp_pixel_rgn_init (&dstPr, drw, 0, 0, width, height, TRUE, TRUE);
+      gimp_pixel_rgn_init (&dstPr, drw, sel_x1, sel_y1, width, height,
+                           TRUE, TRUE);
     }
 
-  gimp_pixel_rgn_get_rect (&srcPr, src, 0, 0, width, height);
+  gimp_pixel_rgn_get_rect (&srcPr, src, sel_x1, sel_y1, width, height);
 
   s = src;
   d = dst;
 
   build_tab (argp->mode);
 
-  gimp_progress_init (_("Hot"));
+  gimp_progress_init (_("Hot..."));
   prog_interval = height / 10;
 
-  for (y = 0; y < height; y++)
+  for (y = sel_y1; y < sel_y2; y++)
     {
       if (y % prog_interval == 0)
-	gimp_progress_update ((double) y / (double) height);
-      for (x = 0; x < width; x++)
+	gimp_progress_update ((double) y / (double) (sel_y2 - sel_y1));
+
+      for (x = sel_x1; x < sel_x2; x++)
 	{
 	  if (hotp (r = *(s + 0), g = *(s + 1), b = *(s + 2)))
 	    {
-	      if (argp->action == act_flag)
+	      if (argp->action == ACT_FLAG)
 		{
 		  for (i = 0; i < 3; i++)
 		    *d++ = 0;
 		  s += 3;
-		  if (Bpp == 4)
+		  if (bpp == 4)
 		    *d++ = *s++;
 		  else if (argp->new_layerp)
 		    *d++ = 255;
@@ -413,7 +409,7 @@ pluginCore (struct piArgs *argp)
 		      *d++ = new_g;
 		      *d++ = new_b;
 		      s += 3;
-		      if (Bpp == 4)
+		      if (bpp == 4)
 			*d++ = *s++;
 		      else if (argp->new_layerp)
 			*d++ = 255;
@@ -456,7 +452,7 @@ pluginCore (struct piArgs *argp)
 		       * pixel with the same luminance (R=G=B=Y), we
 		       * change saturation without affecting luminance.
 		       */
-		      if (argp->action == act_lredux)
+		      if (argp->action == ACT_LREDUX)
 			{
 			  /*
 			   * Calculate a scale factor that will bring the pixel
@@ -481,7 +477,7 @@ pluginCore (struct piArgs *argp)
 			  b = (guint8) pix_encode (scale * pb);
 			}
 		      else
-			{ /* act_sredux hopefully */
+			{ /* ACT_SREDUX hopefully */
 			  /*
 			   * Calculate a scale factor that will bring the
 			   * pixel within both chroma and composite
@@ -501,7 +497,7 @@ pluginCore (struct piArgs *argp)
 			  pr = gc (pr, argp->mode);
 			  pg = gc (pg, argp->mode);
 			  pb = gc (pb, argp->mode);
-			  py = pr * mode[argp->mode].code[0][0] + pg * 
+			  py = pr * mode[argp->mode].code[0][0] + pg *
 			    mode[argp->mode].code[0][1] + pb *
 			    mode[argp->mode].code[0][2];
 			  r = pix_encode (inv_gc (py + scale * (pr - py),
@@ -515,7 +511,7 @@ pluginCore (struct piArgs *argp)
 		      *d++ = new_g = g;
 		      *d++ = new_b = b;
 		      s += 3;
-		      if (Bpp == 4)
+		      if (bpp == 4)
 			*d++ = *s++;
 		      else if (argp->new_layerp)
 			*d++ = 255;
@@ -526,126 +522,110 @@ pluginCore (struct piArgs *argp)
 	    {
 	      if (!argp->new_layerp)
 		{
-		  for (i = 0; i < Bpp; i++)
+		  for (i = 0; i < bpp; i++)
 		    *d++ = *s++;
-		} 
+		}
 	      else
 		{
-		  s += Bpp;
+		  s += bpp;
 		  d += 4;
 		}
 	    }
 	}
     }
-  gimp_pixel_rgn_set_rect (&dstPr, dst, 0, 0, width, height);
+  gimp_pixel_rgn_set_rect (&dstPr, dst, sel_x1, sel_y1, width, height);
 
-  free (src);
-  free (dst);
+  g_free (src);
+  g_free (dst);
 
   if (argp->new_layerp)
     {
       gimp_drawable_flush (ndrw);
-      gimp_drawable_update (nl, 0, 0, width, height);
+      gimp_drawable_update (nl, sel_x1, sel_y1, width, height);
     }
   else
     {
       gimp_drawable_flush (drw);
-      gimp_drawable_merge_shadow (drw->id, TRUE);
-      gimp_drawable_update (drw->id, 0, 0, width, height);
+      gimp_drawable_merge_shadow (drw->drawable_id, TRUE);
+      gimp_drawable_update (drw->drawable_id, sel_x1, sel_y1, width, height);
     }
 
   gimp_displays_flush ();
-  
+
   return retval;
 }
 
-gboolean run_flag = FALSE;
-
-static void
-hot_ok_callback (GtkWidget *widget,
-		 gpointer   data)
-{
-  run_flag = TRUE;
-
-  gtk_widget_destroy (GTK_WIDGET (data));
-}
-
 static gint
-pluginCoreIA (struct piArgs *argp)
+pluginCoreIA (piArgs *argp)
 {
   GtkWidget *dlg;
   GtkWidget *hbox;
   GtkWidget *vbox;
   GtkWidget *toggle;
   GtkWidget *frame;
+  gboolean   run;
 
   gimp_ui_init ("hot", FALSE);
 
   dlg = gimp_dialog_new (_("Hot"), "hot",
-			 gimp_standard_help_func, "filters/hot.html",
-			 GTK_WIN_POS_MOUSE,
-			 FALSE, TRUE, FALSE,
+                         NULL, 0,
+			 gimp_standard_help_func, "plug-in-hot",
 
-			 _("OK"), hot_ok_callback,
-			 NULL, NULL, NULL, TRUE, FALSE,
-			 _("Cancel"), gtk_widget_destroy,
-			 NULL, 1, NULL, FALSE, TRUE,
+			 GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
+			 GTK_STOCK_OK,     GTK_RESPONSE_OK,
 
 			 NULL);
 
-  gtk_signal_connect (GTK_OBJECT (dlg), "destroy",
-		      GTK_SIGNAL_FUNC (gtk_main_quit),
-		      NULL);
-
-  hbox = gtk_hbox_new (FALSE, 5);
-  gtk_container_set_border_width (GTK_CONTAINER (hbox), 5);
+  hbox = gtk_hbox_new (FALSE, 12);
+  gtk_container_set_border_width (GTK_CONTAINER (hbox), 12);
   gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dlg)->vbox), hbox, TRUE, TRUE, 0);
   gtk_widget_show (hbox);
 
-  vbox = gtk_vbox_new (FALSE, 5);
+  vbox = gtk_vbox_new (FALSE, 12);
   gtk_box_pack_start (GTK_BOX (hbox), vbox, TRUE, TRUE, 0);
   gtk_widget_show (vbox);
 
-  toggle = gtk_check_button_new_with_label (_("Create New Layer"));
-  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (toggle), argp->new_layerp);
-  gtk_box_pack_start (GTK_BOX (vbox), toggle, FALSE, FALSE, 0);
-  gtk_signal_connect (GTK_OBJECT (toggle), "toggled",
-		      GTK_SIGNAL_FUNC (gimp_toggle_button_update),
-		      &argp->new_layerp);
-  gtk_widget_show (toggle);
+  frame = gimp_int_radio_group_new (TRUE, _("Mode"),
+				    G_CALLBACK (gimp_radio_button_update),
+				    &argp->mode, argp->mode,
 
-  frame = gimp_radio_group_new2 (TRUE, _("Mode"),
-				 gimp_radio_button_update,
-				 &argp->mode, (gpointer) argp->mode,
+				    "N_TSC", MODE_NTSC, NULL,
+				    "_PAL",  MODE_PAL,  NULL,
 
-				 "NTSC", (gpointer) mode_ntsc, NULL,
-				 "PAL",  (gpointer) mode_pal, NULL,
-
-				 NULL);
+				    NULL);
 
   gtk_box_pack_start (GTK_BOX (vbox), frame, FALSE, FALSE, 0);
   gtk_widget_show (frame);
 
-  frame =
-    gimp_radio_group_new2 (TRUE, _("Action"),
-			   gimp_radio_button_update,
-			   &argp->action, (gpointer) argp->action,
+  toggle = gtk_check_button_new_with_mnemonic (_("Create _New layer"));
+  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (toggle), argp->new_layerp);
+  gtk_box_pack_start (GTK_BOX (vbox), toggle, FALSE, FALSE, 0);
+  gtk_widget_show (toggle);
 
-			   _("Reduce Luminance"),  (gpointer) act_lredux, NULL,
-			   _("Reduce Saturation"), (gpointer) act_sredux, NULL,
-			   _("Blacken"),           (gpointer) act_flag, NULL,
+  g_signal_connect (toggle, "toggled",
+                    G_CALLBACK (gimp_toggle_button_update),
+                    &argp->new_layerp);
 
-			   NULL);
+  frame = gimp_int_radio_group_new (TRUE, _("Action"),
+                                    G_CALLBACK (gimp_radio_button_update),
+                                    &argp->action, argp->action,
+
+                                    _("Reduce _Luminance"),  ACT_LREDUX, NULL,
+                                    _("Reduce _Saturation"), ACT_SREDUX, NULL,
+                                    _("_Blacken"),           ACT_FLAG,   NULL,
+
+                                    NULL);
 
   gtk_box_pack_start (GTK_BOX (hbox), frame, FALSE, FALSE, 0);
   gtk_widget_show (frame);
 
   gtk_widget_show (dlg);
 
-  gtk_main ();
-  gdk_flush ();
+  run = (gimp_dialog_run (GIMP_DIALOG (dlg)) == GTK_RESPONSE_OK);
 
-  if (run_flag)
+  gtk_widget_destroy (dlg);
+
+  if (run)
     return pluginCore (argp);
   else
     return -1;
@@ -674,8 +654,8 @@ pluginCoreIA (struct piArgs *argp)
 static void
 build_tab (int m)
 {
-  register double f;
-  register int pv;
+  double f;
+  int pv;
 
   for (pv = 0; pv <= MAXPIX; pv++)
     {
@@ -700,16 +680,14 @@ build_tab (int m)
   icompos_lim = (int)(compos_lim * SCALE + 0.5);
 }
 
-static int
-hotp (register guint8 r,
-      register guint8 g,
-      register guint8 b)
+static gboolean
+hotp (guint8 r,
+      guint8 g,
+      guint8 b)
 {
-  register int	y, i, q;
-  register long	y2, c2;
+  int	y, i, q;
+  long	y2, c2;
 
-  /*  fprintf(stderr, "\tr: %d, g: %d, b: %d\n", r, g, b);*/
-  
   /*
    * Pixel decoding, gamma correction, and matrix multiplication
    * all done by lookup table.
@@ -749,15 +727,11 @@ hotp (register guint8 r,
   c2 = (long)i * i + (long)q * q;
   y2 = (long)icompos_lim - y;
   y2 *= y2;
-  /*  fprintf(stderr, "hotp: c2: %d; ichroma_lim2: %d; y2: %d; ",
-	  c2, ichroma_lim2, y2);*/
-  
+
   if (c2 <= ichroma_lim2 && c2 <= y2)
     {	/* no problems */
-      /*    fprintf(stderr, "nope\n");*/
-      return 0;
+      return FALSE;
     }
 
-  /*  fprintf(stderr, "yup\n");*/
-  return 1;
+  return TRUE;
 }
