@@ -1,5 +1,5 @@
 /*============================================================================*
- 
+
   Paper Tile 1.0  -- A GIMP PLUG-IN
 
   Copyright (C) 1997-1999 Hirotsuna Mizuno <s1041150@u-aizu.ac.jp>
@@ -18,7 +18,7 @@
   this program; if not, write to the Free Software Foundation, Inc.,
   59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 
- *============================================================================*/
+ *===========================================================================*/
 
 #include "config.h"
 
@@ -31,15 +31,16 @@
 
 #include "libgimp/stdplugins-intl.h"
 
-/*============================================================================*/
-/* DEFINES                                                                    */
-/*============================================================================*/
+/*===========================================================================*/
+/* DEFINES                                                                   */
+/*===========================================================================*/
 
-#define PLUGIN_PROCEDURE_NAME     "plug_in_papertile"
+#define PLUGIN_PROCEDURE_NAME "plug_in_papertile"
+#define HELP_ID               "plug-in-papertile"
 
-/*============================================================================*/
-/* TYPES                                                                      */
-/*============================================================================*/
+/*===========================================================================*/
+/* TYPES                                                                     */
+/*===========================================================================*/
 
 typedef enum
 {
@@ -69,21 +70,21 @@ struct _PluginParams
   gboolean        centering;
   gboolean        wrap_around;
   BackgroundType  background_type;
-  guchar          background_color[4];
+  GimpRGB         background_color;
 };
 
-/*============================================================================*/
-/* VARIABLES                                                                  */
-/*============================================================================*/
+/*===========================================================================*/
+/* VARIABLES                                                                 */
+/*===========================================================================*/
 
 static struct
 {
   PluginParams  params;
 
   gint32        image;
-  GDrawable    *drawable;
+  GimpDrawable *drawable;
   gboolean      drawable_has_alpha;
-  
+
   struct
   {
     gint        x0;
@@ -93,9 +94,9 @@ static struct
     gint        width;
     gint        height;
   } selection;
-  
-  GRunModeType  run_mode;
-  gboolean      run;
+
+  GimpRunMode  run_mode;
+  gboolean         run;
 } p =
 {
   {
@@ -107,7 +108,7 @@ static struct
     TRUE,                       /* centering             */
     FALSE,                      /* wrap_around           */
     BACKGROUND_TYPE_INVERTED,   /* background_type       */
-    { 0, 0, 255, 255 }          /* background_color[4]   */
+    { 0.0, 0.0, 1.0, 1.0 }      /* background_color      */
   },
 
   0,                            /* image                 */
@@ -115,12 +116,12 @@ static struct
   FALSE,                        /* drawable_has_alpha    */
 
   { 0, 0, 0, 0, 0, 0 },         /* selection             */
-  
-  RUN_INTERACTIVE,              /* run_mode              */
+
+  GIMP_RUN_INTERACTIVE,         /* run_mode              */
   FALSE                         /* run                   */
 };
 
-/*----------------------------------------------------------------------------*/
+/*---------------------------------------------------------------------------*/
 
 static void
 params_save_to_gimp (void)
@@ -155,13 +156,13 @@ params_load_from_gimp (void)
 	{
 	  p.params.background_type = BACKGROUND_TYPE_INVERTED;
 	}
-      p.params.background_color[3] = 255;
+      gimp_rgb_set_alpha (&p.params.background_color, 1.0);
     }
 }
 
-/*============================================================================*/
-/* GUI                                                                        */
-/*============================================================================*/
+/*===========================================================================*/
+/* GUI                                                                       */
+/*===========================================================================*/
 
 static struct
 {
@@ -217,15 +218,6 @@ division_y_adj_changed (GtkAdjustment *adj)
   }
 }
 
-static void
-dialog_ok_clicked (GtkWidget *widget,
-		   gpointer   data)
-{
-  p.run = TRUE;
-
-  gtk_widget_destroy (GTK_WIDGET (data));
-}
-
 static void
 open_dialog (void)
 {
@@ -236,186 +228,178 @@ open_dialog (void)
   GtkWidget *vbox;
   GtkWidget *table;
   GtkWidget *frame;
-  GtkWidget *box;
   GtkWidget *color_button;
-  GtkWidget *sep;
 
   gimp_ui_init ("papertile", TRUE);
 
   dialog = gimp_dialog_new (_("Paper Tile"), "papertile",
-			    gimp_standard_help_func, "filters/papertile.html",
-			    GTK_WIN_POS_MOUSE,
-			    FALSE, FALSE, FALSE,
+                            NULL, 0,
+			    gimp_standard_help_func, HELP_ID,
 
-			    _("OK"), dialog_ok_clicked,
-			    NULL, NULL, NULL, TRUE, FALSE,
-			    _("Cancel"), gtk_widget_destroy,
-			    NULL, 1, NULL, FALSE, TRUE,
+			    GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
+			    GTK_STOCK_OK,     GTK_RESPONSE_OK,
 
 			    NULL);
 
-  gtk_signal_connect (GTK_OBJECT (dialog), "destroy",
-		      GTK_SIGNAL_FUNC (gtk_main_quit),
-		      NULL);
-
-  main_hbox = gtk_hbox_new (FALSE, 5);
-  gtk_container_set_border_width (GTK_CONTAINER (main_hbox), 6);
+  main_hbox = gtk_hbox_new (FALSE, 12);
+  gtk_container_set_border_width (GTK_CONTAINER (main_hbox), 12);
   gtk_container_add (GTK_CONTAINER (GTK_DIALOG (dialog)->vbox), main_hbox);
   gtk_widget_show (main_hbox);
 
   /* Left */
-  vbox = gtk_vbox_new (FALSE, 4);
+  vbox = gtk_vbox_new (FALSE, 12);
   gtk_box_pack_start (GTK_BOX (main_hbox), vbox, FALSE, FALSE, 0);
   gtk_widget_show (vbox);
 
-  frame = gtk_frame_new (_("Division"));
+  frame = gimp_frame_new (_("Division"));
   gtk_box_pack_start (GTK_BOX (vbox), frame, FALSE, FALSE, 0);
   gtk_widget_show (frame);
 
   table = gtk_table_new (3, 2, FALSE);
-  gtk_table_set_col_spacings (GTK_TABLE (table), 4);
-  gtk_table_set_row_spacings (GTK_TABLE (table), 2);
-  gtk_container_set_border_width (GTK_CONTAINER (table), 4);
+  gtk_table_set_col_spacings (GTK_TABLE (table), 6);
+  gtk_table_set_row_spacings (GTK_TABLE (table), 6);
   gtk_container_add (GTK_CONTAINER (frame), table);
   gtk_widget_show (table);
 
   button = gimp_spin_button_new (&w.division_x_adj, p.params.division_x,
 				 1.0, p.drawable->width, 1.0, 5.0, 0, 1, 0);
   gimp_table_attach_aligned (GTK_TABLE (table), 0, 0,
-			     _("X:"), 1.0, 0.5,
+			     _("_X:"), 0.0, 0.5,
 			     button, 1, TRUE);
-  gtk_signal_connect (GTK_OBJECT (w.division_x_adj), "value_changed",
-		      GTK_SIGNAL_FUNC (division_x_adj_changed),
-		      NULL);
+  g_signal_connect (w.division_x_adj, "value_changed",
+                    G_CALLBACK (division_x_adj_changed),
+                    NULL);
 
   button = gimp_spin_button_new (&w.division_y_adj, p.params.division_y,
 				 1.0, p.drawable->width, 1.0, 5.0, 0, 1, 0);
   gimp_table_attach_aligned (GTK_TABLE (table), 0, 1,
-			     _("Y:"), 1.0, 0.5,
+			     _("_Y:"), 0.0, 0.5,
 			     button, 1, TRUE);
-  gtk_signal_connect (GTK_OBJECT (w.division_y_adj), "value_changed",
-		      GTK_SIGNAL_FUNC (division_y_adj_changed),
-		      NULL);
+  g_signal_connect (w.division_y_adj, "value_changed",
+                    G_CALLBACK (division_y_adj_changed),
+                    NULL);
 
   button = gimp_spin_button_new (&w.tile_size_adj, p.params.tile_size,
 				 1.0, MAX (p.drawable->width,
 					   p.drawable->height),
 				 1.0, 5.0, 0, 1, 0);
   gimp_table_attach_aligned (GTK_TABLE (table), 0, 2,
-			     _("Size:"), 1.0, 0.5,
+			     _("_Size:"), 0.0, 0.5,
 			     button, 1, TRUE);
-  gtk_signal_connect (GTK_OBJECT (w.tile_size_adj), "value_changed",
-		      GTK_SIGNAL_FUNC (tile_size_adj_changed),
-		      NULL);
+  g_signal_connect (w.tile_size_adj, "value_changed",
+                    G_CALLBACK (tile_size_adj_changed),
+                    NULL);
 
-  frame = gimp_radio_group_new2 (TRUE, _("Fractional Pixels"),
-				 gimp_radio_button_update,
-				 &p.params.fractional_type,
-				 (gpointer) p.params.fractional_type,
+  frame = gimp_int_radio_group_new (TRUE, _("Fractional Pixels"),
+				    G_CALLBACK (gimp_radio_button_update),
+				    &p.params.fractional_type,
+				    p.params.fractional_type,
 
-				 _("Background"),
-				 (gpointer) FRACTIONAL_TYPE_BACKGROUND, NULL,
-				 _("Ignore"),
-				 (gpointer) FRACTIONAL_TYPE_IGNORE, NULL,
-				 _("Force"),
-				 (gpointer) FRACTIONAL_TYPE_FORCE, NULL,
+				    _("_Background"),
+				    FRACTIONAL_TYPE_BACKGROUND, NULL,
+				    _("_Ignore"),
+				    FRACTIONAL_TYPE_IGNORE, NULL,
+				    _("_Force"),
+				    FRACTIONAL_TYPE_FORCE, NULL,
 
-				 NULL);
-  gtk_box_pack_start (GTK_BOX (vbox), frame, TRUE, TRUE, 0);
+				    NULL);
+  gtk_box_pack_start (GTK_BOX (vbox), frame, FALSE, FALSE, 0);
   gtk_widget_show (frame);
 
-  box = GTK_BIN (frame)->child;
-
-  sep = gtk_hseparator_new ();
-  gtk_box_pack_start (GTK_BOX (box), sep, FALSE, FALSE, 1);
-  gtk_widget_show (sep);
-
-  button = gtk_check_button_new_with_label(_("Centering"));
-  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (button), p.params.centering);
-  gtk_box_pack_start (GTK_BOX (box), button, FALSE, FALSE, 0);
-  gtk_signal_connect (GTK_OBJECT (button), "toggled",
-		      GTK_SIGNAL_FUNC (gimp_toggle_button_update),
-		      &p.params.centering);
+  button = gtk_check_button_new_with_mnemonic(_("C_entering"));
+  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (button),
+                                p.params.centering);
+  gtk_box_pack_start (GTK_BOX (vbox), button, FALSE, FALSE, 0);
   gtk_widget_show (button);
 
+  g_signal_connect (button, "toggled",
+                    G_CALLBACK (gimp_toggle_button_update),
+                    &p.params.centering);
+
   /* Right */
-  vbox = gtk_vbox_new (FALSE, 4);
-  gtk_box_pack_start (GTK_BOX (main_hbox), vbox, FALSE, FALSE, 0);
+  vbox = gtk_vbox_new (FALSE, 12);
+  gtk_box_pack_start (GTK_BOX (main_hbox), vbox, TRUE, TRUE, 0);
   gtk_widget_show (vbox);
 
-  frame = gtk_frame_new (_("Movement"));
+  frame = gimp_frame_new (_("Movement"));
   gtk_box_pack_start (GTK_BOX (vbox), frame, FALSE, FALSE, 0);
   gtk_widget_show (frame);
 
   table = gtk_table_new (2, 2, FALSE);
-  gtk_table_set_col_spacings (GTK_TABLE (table), 4);
-  gtk_table_set_row_spacings (GTK_TABLE (table), 2);
-  gtk_container_set_border_width (GTK_CONTAINER (table), 4);
+  gtk_table_set_col_spacings (GTK_TABLE (table), 6);
+  gtk_table_set_row_spacings (GTK_TABLE (table), 6);
   gtk_container_add (GTK_CONTAINER (frame), table);
   gtk_widget_show (table);
 
   button = gimp_spin_button_new (&adjustment, p.params.move_max_rate,
 				 0.0, 100.0, 1.0, 10.0, 0, 1, 0);
   gimp_table_attach_aligned (GTK_TABLE (table), 0, 0,
-			     _("Max (%):"), 1.0, 0.5,
+			     _("_Max (%):"), 0.0, 0.5,
 			     button, 1, TRUE);
-  gtk_signal_connect (GTK_OBJECT (adjustment), "value_changed",
-		      GTK_SIGNAL_FUNC (gimp_double_adjustment_update),
-		      &p.params.move_max_rate);
+  g_signal_connect (adjustment, "value_changed",
+                    G_CALLBACK (gimp_double_adjustment_update),
+                    &p.params.move_max_rate);
 
-  button = gtk_check_button_new_with_label (_("Wrap Around"));
+  button = gtk_check_button_new_with_mnemonic (_("_Wrap around"));
   gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (button),
 				p.params.wrap_around);
   gtk_table_attach_defaults (GTK_TABLE (table), button, 0, 2, 1, 2);
-  gtk_signal_connect (GTK_OBJECT (button), "toggled",
-		      GTK_SIGNAL_FUNC (gimp_toggle_button_update),
-		      &p.params.wrap_around);
   gtk_widget_show (button);
 
-  frame = gimp_radio_group_new2 (TRUE, _("Background Type"),
-				 gimp_radio_button_update,
-				 &p.params.background_type,
-				 (gpointer) p.params.background_type,
+  g_signal_connect (button, "toggled",
+                    G_CALLBACK (gimp_toggle_button_update),
+                    &p.params.wrap_around);
 
-				 _("Transparent"),
-				 (gpointer) BACKGROUND_TYPE_TRANSPARENT, NULL,
-				 _("Inverted Image"),
-				 (gpointer) BACKGROUND_TYPE_INVERTED, NULL,
-				 _("Image"),
-				 (gpointer) BACKGROUND_TYPE_IMAGE, NULL,
-				 _("Foreground Color"),
-				 (gpointer) BACKGROUND_TYPE_FOREGROUND, NULL,
-				 _("Background Color"),
-				 (gpointer) BACKGROUND_TYPE_BACKGROUND, NULL,
-				 (gpointer) 1, /* button without label */
-				 (gpointer) BACKGROUND_TYPE_COLOR, &button,
+  frame = gimp_int_radio_group_new (TRUE, _("Background Type"),
+				    G_CALLBACK (gimp_radio_button_update),
+				    &p.params.background_type,
+				    p.params.background_type,
 
-				 NULL);
+				    _("_Transparent"),
+				    BACKGROUND_TYPE_TRANSPARENT, NULL,
+				    _("I_nverted image"),
+				    BACKGROUND_TYPE_INVERTED, NULL,
+				    _("Im_age"),
+				    BACKGROUND_TYPE_IMAGE, NULL,
+				    _("Fo_reground color"),
+				    BACKGROUND_TYPE_FOREGROUND, NULL,
+				    _("Bac_kground color"),
+				    BACKGROUND_TYPE_BACKGROUND, NULL,
+				    _("S_elect here:"),
+				    BACKGROUND_TYPE_COLOR, &button,
+
+				    NULL);
   gtk_box_pack_start (GTK_BOX (vbox), frame, TRUE, TRUE, 0);
   gtk_widget_show (frame);
 
   color_button = gimp_color_button_new (_("Background Color"), 100, 16,
-					p.params.background_color,
-					p.drawable_has_alpha ? 4 : 3);
-  gtk_container_add (GTK_CONTAINER (button), color_button);
+					&p.params.background_color,
+					p.drawable_has_alpha ?
+					GIMP_COLOR_AREA_SMALL_CHECKS :
+					GIMP_COLOR_AREA_FLAT);
+  gtk_box_pack_start (GTK_BOX (GTK_BIN (frame)->child),
+                      color_button, TRUE, TRUE, 0);
   gtk_widget_show (color_button);
 
   gtk_widget_set_sensitive (color_button,
 			    p.params.background_type == BACKGROUND_TYPE_COLOR);
-  gtk_object_set_data (GTK_OBJECT (button), "set_sensitive", color_button);
+  g_object_set_data (G_OBJECT (button), "set_sensitive", color_button);
+
+  g_signal_connect (color_button, "color_changed",
+                    G_CALLBACK (gimp_color_button_get_color),
+                    &p.params.background_color);
 
   gtk_widget_show (dialog);
 
-  gtk_main ();
-  gdk_flush ();
+  p.run = (gimp_dialog_run (GIMP_DIALOG (dialog)) == GTK_RESPONSE_OK);
+
+  gtk_widget_destroy (dialog);
 }
 
-/*============================================================================*/
-/* PLUGIN CORE                                                                */
-/*============================================================================*/
-
-typedef struct _Tile Tile;
-struct _Tile
+/*===========================================================================*/
+/* PLUGIN CORE                                                               */
+/*===========================================================================*/
+typedef struct _Tile
 {
   guint x;
   guint y;
@@ -424,7 +408,7 @@ struct _Tile
   guint height;
   gint  move_x;
   gint  move_y;
-};
+} Tile;
 
 static gint
 tile_compare (const void *x,
@@ -433,13 +417,10 @@ tile_compare (const void *x,
   return ((Tile *) x)->z - ((Tile *) y)->z;
 }
 
-/*----------------------------------------------------------------------------*/
-
 static inline gdouble
 drand (void)
 {
-  static gdouble R = 2.0 / (gdouble) G_MAXRAND;
-  return (gdouble) rand () * R - 1.0;
+  return g_random_double_range (-1, 1);
 }
 
 static inline void
@@ -453,8 +434,6 @@ random_move (gint *x,
   *y = (gint) (radius * sin (angle));
 }
 
-/*----------------------------------------------------------------------------*/
-
 static void
 overlap_RGB (guchar       *base,
 	     const guchar *top)
@@ -499,14 +478,13 @@ overlap_RGBA (guchar       *base,
     }
 }
 
-/*----------------------------------------------------------------------------*/
-
 static inline void
 filter (void)
 {
   static void (* overlap)(guchar *, const guchar *);
-  GPixelRgn  src;
-  GPixelRgn  dst;
+  GimpPixelRgn  src;
+  GimpPixelRgn  dst;
+  GimpRGB       color;
   guchar     pixel[4];
   gint       division_x;
   gint       division_y;
@@ -519,7 +497,6 @@ filter (void)
   gint       x;
   gint       y;
   gint       move_max_pixels;
-  guchar    *row_buffer;
   gint       clear_x0;
   gint       clear_y0;
   gint       clear_x1;
@@ -531,13 +508,15 @@ filter (void)
   gint       dindex;
   gint       sindex;
   gint       px, py;
+  GRand     *gr;
+
+  gr = g_rand_new ();
 
   /* INITIALIZE */
   gimp_pixel_rgn_init (&src, p.drawable, 0, 0,
 		       p.drawable->width, p.drawable->height, FALSE, FALSE);
   gimp_pixel_rgn_init (&dst, p.drawable, 0, 0,
 		       p.drawable->width, p.drawable->height, TRUE, TRUE);
-  row_buffer = g_new (guchar, p.drawable->bpp * p.drawable->width);
   pixels = g_new (guchar,
 		  p.drawable->bpp * p.drawable->width * p.drawable->height);
   buffer = g_new (guchar,
@@ -545,10 +524,17 @@ filter (void)
 
   overlap = p.drawable_has_alpha ? overlap_RGBA : overlap_RGB;
 
-  gimp_progress_init ("Paper Tile...");
+  gimp_progress_init (_("Paper Tile..."));
+
+  gimp_drawable_mask_bounds (p.drawable->drawable_id,
+			     &p.selection.x0, &p.selection.y0,
+			     &p.selection.x1, &p.selection.y1);
+  p.selection.width  = p.selection.x1 - p.selection.x0;
+  p.selection.height = p.selection.y1 - p.selection.y0;
+
+  gimp_tile_cache_ntiles (2 * (p.selection.width / gimp_tile_width () + 1));
 
   /* TILES */
-  srand (0);
   division_x = p.params.division_x;
   division_y = p.params.division_y;
   if (p.params.fractional_type == FRACTIONAL_TYPE_FORCE)
@@ -638,19 +624,15 @@ filter (void)
 	      t->y      = srcy;
 	      t->height = p.drawable->height - srcy;
 	    }
-	  t->z = rand ();
+	  t->z = g_rand_int (gr);
 	  random_move (&t->move_x, &t->move_y, move_max_pixels);
 	}
     }
   qsort (tiles, numof_tiles, sizeof *tiles, tile_compare);
 
-  /* CLEAR PIXELS */
-  for (y = 0; y < p.drawable->height; y++)
-    {
-      gimp_pixel_rgn_get_row (&src,
-			      &pixels[p.drawable->bpp*p.drawable->width*y],
-			      0, y, p.drawable->width);
-    }
+  gimp_pixel_rgn_get_rect (&src, pixels, 0, 0, p.drawable->width,
+			   p.drawable->height);
+
   if (p.params.fractional_type == FRACTIONAL_TYPE_IGNORE)
     {
       clear_x0     = offset_x;
@@ -701,7 +683,8 @@ filter (void)
       break;
 
     case BACKGROUND_TYPE_FOREGROUND:
-      gimp_palette_get_foreground (&pixel[0], &pixel[1], &pixel[2]);
+      gimp_context_get_foreground (&color);
+      gimp_rgb_get_uchar (&color, &pixel[0], &pixel[1], &pixel[2]);
       pixel[3] = 255;
       for (y = clear_y0; y < clear_y1; y++)
 	{
@@ -717,7 +700,8 @@ filter (void)
       break;
 
     case BACKGROUND_TYPE_BACKGROUND:
-      gimp_palette_get_background (&pixel[0], &pixel[1], &pixel[2]);
+      gimp_context_get_background (&color);
+      gimp_rgb_get_uchar (&color, &pixel[0], &pixel[1], &pixel[2]);
       pixel[3] = 255;
       for (y = clear_y0; y < clear_y1; y++)
 	{
@@ -733,10 +717,8 @@ filter (void)
       break;
 
     case BACKGROUND_TYPE_COLOR:
-      pixel[0] = p.params.background_color[0];
-      pixel[1] = p.params.background_color[1];
-      pixel[2] = p.params.background_color[2];
-      pixel[3] = p.params.background_color[3];
+      gimp_rgba_get_uchar (&p.params.background_color,
+			   pixel, pixel + 1, pixel + 2, pixel + 3);
       for (y = clear_y0; y < clear_y1; y++)
 	{
 	  for (x = clear_x0; x < clear_x1; x++)
@@ -782,28 +764,22 @@ filter (void)
 	    }
 	}
 
-      gimp_progress_update ((gdouble) i / (gdouble) numof_tiles / 2.0);
+      gimp_progress_update ((gdouble) i / (gdouble) numof_tiles);
     }
 
-  /* SEND IT */
-  for (y = 0; y < p.drawable->height; y++)
-    {
-      gimp_pixel_rgn_set_row (&dst, &pixels[p.drawable->bpp*p.drawable->width*y],
-			      0, y, p.drawable->width);
-      gimp_progress_update ((gdouble) y /
-			    (gdouble) p.drawable->height / 2.0 + 0.5);
-    }
+  gimp_pixel_rgn_set_rect (&dst, pixels, 0, 0, p.drawable->width,
+			   p.drawable->height);
 
   gimp_drawable_flush (p.drawable);
-  gimp_drawable_merge_shadow (p.drawable->id, TRUE);
-  gimp_drawable_update (p.drawable->id, p.selection.x0, p.selection.y0,
+  gimp_drawable_merge_shadow (p.drawable->drawable_id, TRUE);
+  gimp_drawable_update (p.drawable->drawable_id,
+                        p.selection.x0, p.selection.y0,
 			p.selection.width, p.selection.height);
 
-  /* FREE */
+  g_rand_free (gr);
   g_free (buffer);
   g_free (pixels);
   g_free (tiles);
-  g_free (row_buffer);
 }
 
 /*============================================================================*/
@@ -813,93 +789,91 @@ filter (void)
 static void
 plugin_query (void)
 {
-  static GParamDef     args[]            =
+  static GimpParamDef     args[]            =
   {
-    { PARAM_INT32,    "run_mode",         "run mode"                         },
-    { PARAM_IMAGE,    "image",            "input image"                      },
-    { PARAM_DRAWABLE, "drawable",         "input drawable"                   },
-    { PARAM_INT32,    "tile_size",        "tile size (pixels)"               },
-    { PARAM_FLOAT,    "move_max",         "max move rate (%)"                },
-    { PARAM_INT32,    "fractional_type",  "0:Background 1:Ignore 2:Force"    },
-    { PARAM_INT32,    "wrap_around",      "wrap around (bool)"               },
-    { PARAM_INT32,    "centering",        "centering (bool)"                 },
-    { PARAM_INT32,    "background_type",
+    { GIMP_PDB_INT32,    "run_mode",         "run mode"                         },
+    { GIMP_PDB_IMAGE,    "image",            "input image"                      },
+    { GIMP_PDB_DRAWABLE, "drawable",         "input drawable"                   },
+    { GIMP_PDB_INT32,    "tile_size",        "tile size (pixels)"               },
+    { GIMP_PDB_FLOAT,    "move_max",         "max move rate (%)"                },
+    { GIMP_PDB_INT32,    "fractional_type",  "0:Background 1:Ignore 2:Force"    },
+    { GIMP_PDB_INT32,    "wrap_around",      "wrap around (bool)"               },
+    { GIMP_PDB_INT32,    "centering",        "centering (bool)"                 },
+    { GIMP_PDB_INT32,    "background_type",
       "0:Transparent 1:Inverted 2:Image? 3:FG 4:BG 5:Color"                  },
-    { PARAM_INT32,    "background_color", "background color (for bg-type 5)" },
-    { PARAM_INT32,    "background_alpha", "opacity (for bg-type 5)"          }
+    { GIMP_PDB_INT32,    "background_color", "background color (for bg-type 5)" },
+    { GIMP_PDB_INT32,    "background_alpha", "opacity (for bg-type 5)"          }
   };
-  static gint numof_args        = sizeof args / sizeof args[0];
 
   gimp_install_procedure (PLUGIN_PROCEDURE_NAME,
-			  "Cuts an image into paper tiles, and slides each paper tile.",
-			  "This plug-in cuts an image into paper tiles and slides each paper tile.",
+			  "Cuts an image into paper tiles, and slides each "
+                          "paper tile.",
+			  "This plug-in cuts an image into paper tiles and "
+                          "slides each paper tile.",
 			  "Hirotsuna Mizuno <s1041150@u-aizu.ac.jp>",
 			  "Copyright (c)1997-1999 Hirotsuna Mizuno",
 			  _("September 31, 1999"),
-			  N_("<Image>/Filters/Map/Paper Tile..."),
+			  N_("_Paper Tile..."),
 			  "RGB*",
-			  PROC_PLUG_IN,
-			  numof_args, 0,
+			  GIMP_PLUGIN,
+			  G_N_ELEMENTS (args), 0,
 			  args, NULL);
+
+  gimp_plugin_menu_register (PLUGIN_PROCEDURE_NAME, "<Image>/Filters/Map");
 }
 
 static void
-plugin_run (gchar   *name,
-	    gint     numof_params,
-	    GParam  *params,
-	    gint    *numof_return_vals,
-	    GParam **return_vals)
+plugin_run (const gchar      *name,
+	    gint              numof_params,
+	    const GimpParam  *params,
+	    gint             *numof_return_vals,
+	    GimpParam       **return_vals)
 {
-  GStatusType status;
+  GimpPDBStatusType status;
 
-  INIT_I18N_UI();
+  INIT_I18N ();
 
-  status = STATUS_SUCCESS;
+  status = GIMP_PDB_SUCCESS;
   p.run  = FALSE;
   p.run_mode = params[0].data.d_int32;
   p.image    = params[1].data.d_image;
   p.drawable = gimp_drawable_get(params[2].data.d_drawable);
-  p.drawable_has_alpha = gimp_drawable_has_alpha(p.drawable->id);
+  p.drawable_has_alpha = gimp_drawable_has_alpha(p.drawable->drawable_id);
 
-  gimp_drawable_mask_bounds (p.drawable->id,
-			     &p.selection.x0, &p.selection.y0,
-			     &p.selection.x1, &p.selection.y1);
-  p.selection.width  = p.selection.x1 - p.selection.x0;
-  p.selection.height = p.selection.y1 - p.selection.y0;
-
-  if (gimp_drawable_is_rgb (p.drawable->id))
+  if (gimp_drawable_is_rgb (p.drawable->drawable_id))
     {
       switch (p.run_mode)
 	{
-	case RUN_INTERACTIVE:
+	case GIMP_RUN_INTERACTIVE:
 	  params_load_from_gimp ();
 	  open_dialog ();
 	  break;
 
-	case RUN_NONINTERACTIVE:
+	case GIMP_RUN_NONINTERACTIVE:
 	  if (numof_params == 11)
 	    {
-	      p.params.tile_size  = params[3].data.d_int32;
-	      p.params.division_x = p.drawable->width  / p.params.tile_size;
-	      p.params.division_y = p.drawable->height / p.params.tile_size;
-	      p.params.move_max_rate = params[4].data.d_float;
-	      p.params.fractional_type = (FractionalType)params[5].data.d_int32;
-	      p.params.wrap_around = params[6].data.d_int32;
-	      p.params.centering = params[7].data.d_int32;
-	      p.params.background_type = (BackgroundType)params[8].data.d_int32;
-	      p.params.background_color[0] = params[9].data.d_color.red;
-	      p.params.background_color[1] = params[9].data.d_color.green;
-	      p.params.background_color[2] = params[9].data.d_color.blue;
-	      p.params.background_color[3] = params[10].data.d_int32;
+	      p.params.tile_size        = params[3].data.d_int32;
+	      p.params.division_x       = p.drawable->width  / p.params.tile_size;
+	      p.params.division_y       = p.drawable->height / p.params.tile_size;
+	      p.params.move_max_rate    = params[4].data.d_float;
+	      p.params.fractional_type  = (FractionalType)params[5].data.d_int32;
+	      p.params.wrap_around      = params[6].data.d_int32;
+	      p.params.centering        = params[7].data.d_int32;
+	      p.params.background_type  = (BackgroundType)params[8].data.d_int32;
+	      p.params.background_color = params[9].data.d_color;
+
+	      /*  FIXME:  this used to be the alpha value
+				          params[10].data.d_int32
+	       */
 	      p.run = TRUE;
 	    }
 	  else
 	    {
-	      status = STATUS_CALLING_ERROR;
+	      status = GIMP_PDB_CALLING_ERROR;
 	    }
 	  break;
 
-	case RUN_WITH_LAST_VALS:
+	case GIMP_RUN_WITH_LAST_VALS:
 	  params_load_from_gimp ();
 	  p.run = TRUE;
 	  break;
@@ -907,36 +881,31 @@ plugin_run (gchar   *name,
     }
   else
     {
-      status = STATUS_EXECUTION_ERROR;
+      status = GIMP_PDB_EXECUTION_ERROR;
     }
 
-  if (status == STATUS_SUCCESS && p.run)
+  if (status == GIMP_PDB_SUCCESS && p.run)
     {
-      if(p.run_mode == RUN_INTERACTIVE)
-	{
-	  params_save_to_gimp ();
-	  gimp_undo_push_group_start (p.image);
-	}
+      params_save_to_gimp ();
 
       filter ();
 
-      if (p.run_mode == RUN_INTERACTIVE)
-	{
-	  gimp_undo_push_group_end (p.image);
-	  gimp_displays_flush ();
-	}
+      if (p.run_mode != GIMP_RUN_NONINTERACTIVE)
+        gimp_displays_flush ();
     }
-  
+
+  gimp_drawable_detach (p.drawable);
+
   {
-    static GParam return_value[1];
-    return_value[0].type          = PARAM_STATUS;
+    static GimpParam return_value[1];
+    return_value[0].type          = GIMP_PDB_STATUS;
     return_value[0].data.d_status = status;
     *numof_return_vals            = 1;
     *return_vals                  = return_value;
   }
 }
 
-GPlugInInfo PLUG_IN_INFO =
+GimpPlugInInfo PLUG_IN_INFO =
 {
   NULL,
   NULL,
