@@ -39,6 +39,8 @@
 #include <glib.h>		/* We want glib.h first because of some
 				 * pretty obscure Win32 compilation issues.
 				 */
+
+#include <errno.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -50,9 +52,10 @@
 
 #include <libgimp/gimp.h>
 #include <libgimp/gimpui.h>
-#include <libgimp/parasiteio.h>
+#include <libgimpbase/gimpparasiteio.h>
+
+#include "libgimp/stdplugins-intl.h"
 
-#include <libgimp/stdplugins-intl.h>
 
 /* Note that the upcoming PSP version 6 writes PSP file format version
  * 4.0, but the documentation for that apparently isn't publicly
@@ -286,19 +289,19 @@ typedef struct
 /* Declare some local functions.
  */
 static void   query      (void);
-static void   run        (gchar   *name,
-                          gint     nparams,
-                          GParam  *param,
-                          gint    *nreturn_vals,
-                          GParam **return_vals);
-static gint32 load_image (gchar  *filename);
-static gint   save_image (gchar  *filename,
-			  gint32  image_ID,
-			  gint32  drawable_ID);
+static void   run        (const gchar      *name,
+                          gint              nparams,
+                          const GimpParam  *param,
+                          gint             *nreturn_vals,
+                          GimpParam       **return_vals);
+static gint32 load_image (const gchar      *filename);
+static gint   save_image (const gchar      *filename,
+			  gint32            image_ID,
+			  gint32            drawable_ID);
 
 /* Various local variables...
  */
-GPlugInInfo PLUG_IN_INFO =
+GimpPlugInInfo PLUG_IN_INFO =
 {
   NULL,  /* init_proc  */
   NULL,  /* quit_proc  */
@@ -312,56 +315,40 @@ typedef struct
   PSPCompression compression;
 } PSPSaveVals;
 
-typedef struct
-{
-  gint  run;
-} PSPSaveInterface;
-
 static PSPSaveVals psvals =
 {
   PSP_COMP_LZ77
 };
 
-static PSPSaveInterface psint =
-{
-  FALSE   /* run */
-};
-
 static guint16 major, minor;
-static guint tile_height;
 
-MAIN()
+
+MAIN ()
 
 static void
 query (void)
 {
-  static GParamDef load_args[] =
+  static GimpParamDef load_args[] =
   {
-    { PARAM_INT32, "run_mode", "Interactive, non-interactive" },
-    { PARAM_STRING, "filename", "The name of the file to load" },
-    { PARAM_STRING, "raw_filename", "The name of the file to load" },
+    { GIMP_PDB_INT32, "run_mode", "Interactive, non-interactive" },
+    { GIMP_PDB_STRING, "filename", "The name of the file to load" },
+    { GIMP_PDB_STRING, "raw_filename", "The name of the file to load" }
   };
-  static GParamDef load_return_vals[] =
+  static GimpParamDef load_return_vals[] =
   {
-    { PARAM_IMAGE, "image", "Output image" },
+    { GIMP_PDB_IMAGE, "image", "Output image" }
   };
-  static gint nload_args = sizeof (load_args) / sizeof (load_args[0]);
-  static gint nload_return_vals = (sizeof (load_return_vals) /
-				   sizeof (load_return_vals[0]));
 
-/*    static GParamDef save_args[] = */
+/*    static GimpParamDef save_args[] = */
 /*    { */
-/*      { PARAM_INT32, "run_mode", "Interactive, non-interactive" }, */
-/*      { PARAM_IMAGE, "image", "Input image" }, */
-/*      { PARAM_DRAWABLE, "drawable", "Drawable to save" }, */
-/*      { PARAM_STRING, "filename", "The name of the file to save the image in" }, */
-/*      { PARAM_STRING, "raw_filename", "The name of the file to save the image in" }, */
-/*      { PARAM_INT32, "compression", "Specify 0 for no compression, " */
+/*      { GIMP_PDB_INT32, "run_mode", "Interactive, non-interactive" }, */
+/*      { GIMP_PDB_IMAGE, "image", "Input image" }, */
+/*      { GIMP_PDB_DRAWABLE, "drawable", "Drawable to save" }, */
+/*      { GIMP_PDB_STRING, "filename", "The name of the file to save the image in" }, */
+/*      { GIMP_PDB_STRING, "raw_filename", "The name of the file to save the image in" }, */
+/*      { GIMP_PDB_INT32, "compression", "Specify 0 for no compression, " */
 /*        "1 for RLE, and 2 for LZ77" } */
 /*    }; */
-/*    static gint nsave_args = sizeof (save_args) / sizeof (save_args[0]); */
-
-  INIT_I18N();
 
   gimp_install_procedure ("file_psp_load",
                           "loads images from the Paint Shop Pro PSP file format",
@@ -372,12 +359,19 @@ query (void)
 			  "Tor Lillqvist",
                           "Tor Lillqvist",
                           "1999",
-                          "<Load>/PSP",
+                          N_("Paint Shop Pro image"),
 			  NULL,
-                          PROC_PLUG_IN,
-                          nload_args, nload_return_vals,
+                          GIMP_PLUGIN,
+                          G_N_ELEMENTS (load_args),
+                          G_N_ELEMENTS (load_return_vals),
                           load_args, load_return_vals);
 
+  gimp_register_file_handler_mime ("file_psp_load", "image/x-psp");
+  gimp_register_magic_load_handler ("file_psp_load",
+				    "psp,tub",
+				    "",
+				    "0,string,Paint\\040Shop\\040Pro\\040Image\\040File\n\032");
+
 /* Removed until Saving is implemented -- njl195@zepler.org
   gimp_install_procedure ("file_psp_save",
                           "saves images in the Paint Shop Pro PSP file format",
@@ -388,90 +382,54 @@ query (void)
                           "Tor Lillqvist",
                           "Tor Lillqvist",
                           "1999",
-                          "<Save>/PSP",
+                          N_("Paint Shop Pro image"),
 			  "RGB*, GRAY*, INDEXED*",
-                          PROC_PLUG_IN,
-                          nsave_args, 0,
+                          GIMP_PLUGIN,
+                          G_N_ELEMENTS (save_args), 0,
                           save_args, NULL);
-*/
 
-  gimp_register_magic_load_handler ("file_psp_load",
-				    "psp,tub",
-				    "",
-				    "0,string,Paint Shop Pro Image File\n\032");
-/* Removed until Saving is implemented -- njl195@zepler.org
-  gimp_register_save_handler       ("file_psp_save",
-                                    "psp,tub",
-                                    "");
+  gimp_register_save_handler ("file_psp_save", "psp,tub", "");
 */
 }
 
-static void
-save_ok_callback (GtkWidget *widget,
-		  gpointer   data)
-{
-  psint.run = TRUE;
-
-  gtk_widget_destroy (GTK_WIDGET (data));
-}
-
-static void 
-init_gtk (void)
-{
-  gchar **argv;
-  gint    argc;
-
-  argc    = 1;
-  argv    = g_new (gchar *, 1);
-  argv[0] = g_strdup ("psp");
-  
-  gtk_init (&argc, &argv);
-  gtk_rc_parse (gimp_gtkrc ());
-}
-
-static gint
+static gboolean
 save_dialog (void)
 {
   GtkWidget *dlg;
   GtkWidget *frame;
+  gint       run;
 
   dlg = gimp_dialog_new (_("Save as PSP"), "psp",
-			 gimp_plugin_help_func, "filters/psp.html",
-			 GTK_WIN_POS_MOUSE,
-			 FALSE, TRUE, FALSE,
+                         NULL, 0,
+			 gimp_standard_help_func, "file-psp-load",
 
-			 _("OK"), save_ok_callback,
-			 NULL, NULL, NULL, TRUE, FALSE,
-			 _("Cancel"), gtk_widget_destroy,
-			 NULL, 1, NULL, FALSE, TRUE,
+			 GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
+			 GTK_STOCK_OK,     GTK_RESPONSE_OK,
 
 			 NULL);
 
-  gtk_signal_connect (GTK_OBJECT (dlg), "destroy",
-		      GTK_SIGNAL_FUNC (gtk_main_quit),
-		      NULL);
-
   /*  file save type  */
-  frame = gimp_radio_group_new2 (TRUE, _("Data Compression"),
-				 gimp_radio_button_update,
-				 &psvals.compression,
-				 (gpointer) psvals.compression,
+  frame = gimp_int_radio_group_new (TRUE, _("Data Compression"),
+				    G_CALLBACK (gimp_radio_button_update),
+				    &psvals.compression, psvals.compression,
 
-				 _("None"), (gpointer) PSP_COMP_NONE, NULL,
-				 _("RLE"),  (gpointer) PSP_COMP_RLE, NULL,
-				 _("LZ77"), (gpointer) PSP_COMP_LZ77, NULL,
+				    _("None"), PSP_COMP_NONE, NULL,
+				    _("RLE"),  PSP_COMP_RLE,  NULL,
+				    _("LZ77"), PSP_COMP_LZ77, NULL,
 
-				 NULL);
-  gtk_container_set_border_width (GTK_CONTAINER (frame), 6);
+				    NULL);
+
+  gtk_container_set_border_width (GTK_CONTAINER (frame), 12);
   gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dlg)->vbox), frame, FALSE, TRUE, 0);
   gtk_widget_show (frame);
 
   gtk_widget_show (dlg);
 
-  gtk_main ();
-  gdk_flush ();
+  run = (gimp_dialog_run (GIMP_DIALOG (dlg)) == GTK_RESPONSE_OK);
 
-  return psint.run;
+  gtk_widget_destroy (dlg);
+
+  return run;
 }
 
 static gchar *
@@ -520,22 +478,22 @@ read_block_header (FILE    *f,
       || fread (&len, 4, 1, f) < 1
       || (major < 4 && fread (total_len, 4, 1, f) < 1))
     {
-      g_message ("PSP: Error reading block header");
+      g_message ("Error reading block header");
       fclose (f);
       return -1;
     }
   if (memcmp (buf, "~BK\0", 4) != 0)
     {
-      IFDBG(3) 
-	g_message ("PSP: Invalid block header at %ld", header_start);
+      IFDBG(3)
+	g_message ("Invalid block header at %ld", header_start);
       else
-	g_message ("PSP: Invalid block header");
-    
+	g_message ("Invalid block header");
+
       fclose (f);
       return -1;
     }
 
-  IFDBG(3) g_message ("PSP: %s at %ld", block_name (id), header_start);
+  IFDBG(3) g_message ("%s at %ld", block_name (id), header_start);
 
   if (major < 4)
     {
@@ -551,7 +509,7 @@ read_block_header (FILE    *f,
       *total_len = GUINT32_FROM_LE (len);
     }
 
-  return id;
+  return GUINT16_FROM_LE (id);
 }
 
 static gint
@@ -561,25 +519,21 @@ read_general_image_attribute_block (FILE     *f,
 				    PSPimage *ia)
 {
   gchar buf[6];
-#ifdef G_HAVE_GINT64
-  gint64 res[1];
-#else
-  guchar res[8];
-#endif
+  guint64 res;
 
   if (init_len < 38 || total_len < 38)
     {
-      g_message ("PSP: Invalid general image attribute chunk size");
+      g_message ("Invalid general image attribute chunk size");
       fclose (f);
       return -1;
     }
 
   if (major >= 4)
     fseek (f, 4, SEEK_CUR);
-  
+
   if (fread (&ia->width, 4, 1, f) < 1
       || fread (&ia->height, 4, 1, f) < 1
-      || fread (res, 8, 1, f) < 1
+      || fread (&res, 8, 1, f) < 1
       || fread (&ia->metric, 1, 1, f) < 1
       || fread (&ia->compression, 2, 1, f) < 1
       || fread (&ia->depth, 2, 1, f) < 1
@@ -589,43 +543,22 @@ read_general_image_attribute_block (FILE     *f,
       || fread (&ia->active_layer, 4, 1, f) < 1
       || fread (&ia->layer_count, 2, 1, f) < 1)
     {
-      g_message ("PSP: Error reading general image attribute block");
+      g_message ("Error reading general image attribute block");
       fclose (f);
       return -1;
     }
   ia->width = GUINT32_FROM_LE (ia->width);
   ia->height = GUINT32_FROM_LE (ia->height);
 
-#ifdef G_HAVE_GINT64
-  res[0] = GUINT64_FROM_LE (res[0]);
-#else
-#if G_BYTE_ORDER == G_BIG_ENDIAN
-  {
-    /* Swap bytes in the double */
-    guchar t;
-    t = res[0];
-    res[0] = res[7];
-    res[7] = t;
-    t = res[1];
-    res[1] = res[6];
-    res[6] = t;
-    t = res[2];
-    res[2] = res[5];
-    res[5] = t;
-    t = res[3];
-    res[3] = res[4];
-    res[4] = t;
-  }
-#endif
-#endif
-  memcpy (&ia->resolution, res, 8);
+  res = GUINT64_FROM_LE (res);
+  memcpy (&ia->resolution, &res, 8);
   if (ia->metric == PSP_METRIC_CM)
     ia->resolution /= 2.54;
 
   ia->compression = GUINT16_FROM_LE (ia->compression);
   if (ia->compression > PSP_COMP_LZ77)
     {
-      g_message ("PSP: Unknown compression type %d", ia->compression);
+      g_message ("Unknown compression type %d", ia->compression);
       fclose (f);
       return -1;
     }
@@ -633,7 +566,7 @@ read_general_image_attribute_block (FILE     *f,
   ia->depth = GUINT16_FROM_LE (ia->depth);
   if (ia->depth != 24)
     {
-      g_message ("PSP: Unsupported bit depth %d", ia->depth);
+      g_message ("Unsupported bit depth %d", ia->depth);
       fclose (f);
       return -1;
     }
@@ -651,7 +584,7 @@ try_fseek (FILE  *f,
 {
   if (fseek (f, pos, whence) < 0)
     {
-      g_message ("PSP: Seek error");
+      g_message ("Seek error: %s", g_strerror (errno));
       fclose (f);
       return -1;
     }
@@ -673,7 +606,7 @@ read_creator_block (FILE     *f,
   guint32 dword;
   guint32 cdate = 0, mdate = 0, appid, appver;
   GString *comment;
-  Parasite *comment_parasite;
+  GimpParasite *comment_parasite;
 
   data_start = ftell (f);
   comment = g_string_new (NULL);
@@ -684,14 +617,14 @@ read_creator_block (FILE     *f,
 	  || fread (&keyword, 2, 1, f) < 1
 	  || fread (&length, 4, 1, f) < 1)
 	{
-	  g_message ("PSP: Error reading creator keyword chunk");
+	  g_message ("Error reading creator keyword chunk");
 	  fclose (f);
 	  gimp_image_delete (image_ID);
 	  return -1;
 	}
       if (memcmp (buf, "~FL\0", 4) != 0)
 	{
-	  g_message ("PSP: Invalid keyword chunk header");
+	  g_message ("Invalid keyword chunk header");
 	  fclose (f);
 	  gimp_image_delete (image_ID);
 	  return -1;
@@ -707,7 +640,7 @@ read_creator_block (FILE     *f,
 	  string = g_malloc (length + 1);
 	  if (fread (string, length, 1, f) < 1)
 	    {
-	      g_message ("PSP: Error reading creator keyword data");
+	      g_message ("Error reading creator keyword data");
 	      fclose (f);
 	      gimp_image_delete (image_ID);
 	      return -1;
@@ -732,7 +665,7 @@ read_creator_block (FILE     *f,
 	case PSP_CRTR_FLD_APP_VER:
 	  if (fread (&dword, 4, 1, f) < 1)
 	    {
-	      g_message ("PSP: Error reading creator keyword data");
+	      g_message ("Error reading creator keyword data");
 	      fclose (f);
 	      gimp_image_delete (image_ID);
 	      return -1;
@@ -786,10 +719,12 @@ read_creator_block (FILE     *f,
     }
   if (comment->len > 0)
     {
-      comment_parasite = parasite_new("gimp-comment", PARASITE_PERSISTENT,
-				      strlen (comment->str) + 1, comment->str);
+      comment_parasite = gimp_parasite_new ("gimp-comment",
+					    GIMP_PARASITE_PERSISTENT,
+					    strlen (comment->str) + 1,
+					    comment->str);
       gimp_image_parasite_attach(image_ID, comment_parasite);
-      parasite_free (comment_parasite);
+      gimp_parasite_free (comment_parasite);
     }
 
   g_string_free (comment, FALSE);
@@ -806,38 +741,38 @@ swab_rect (guint32 *rect)
   rect[3] = GUINT32_FROM_LE (rect[3]);
 }
 
-static GLayerMode
+static GimpLayerModeEffects
 gimp_layer_mode_from_psp_blend_mode (PSPLayerBlendModes mode)
 {
   switch (mode)
     {
     case PSP_BLEND_NORMAL:
-      return NORMAL_MODE;
+      return GIMP_NORMAL_MODE;
     case PSP_BLEND_DARKEN:
-      return DARKEN_ONLY_MODE;
+      return GIMP_DARKEN_ONLY_MODE;
     case PSP_BLEND_LIGHTEN:
-      return LIGHTEN_ONLY_MODE;
+      return GIMP_LIGHTEN_ONLY_MODE;
     case PSP_BLEND_HUE:
-      return HUE_MODE;
+      return GIMP_HUE_MODE;
     case PSP_BLEND_SATURATION:
-      return SATURATION_MODE;
+      return GIMP_SATURATION_MODE;
     case PSP_BLEND_COLOR:
-      return COLOR_MODE;
+      return GIMP_COLOR_MODE;
     case PSP_BLEND_LUMINANCE:
-      return VALUE_MODE;	/* ??? */
+      return GIMP_VALUE_MODE;	/* ??? */
     case PSP_BLEND_MULTIPLY:
-      return MULTIPLY_MODE;
+      return GIMP_MULTIPLY_MODE;
     case PSP_BLEND_SCREEN:
-      return SCREEN_MODE;
+      return GIMP_SCREEN_MODE;
     case PSP_BLEND_DISSOLVE:
-      return DISSOLVE_MODE;
+      return GIMP_DISSOLVE_MODE;
     case PSP_BLEND_OVERLAY:
-      return OVERLAY_MODE;
+      return GIMP_OVERLAY_MODE;
     case PSP_BLEND_HARD_LIGHT:
     case PSP_BLEND_SOFT_LIGHT:
       return -1;
     case PSP_BLEND_DIFFERENCE:
-      return DIFFERENCE_MODE;
+      return GIMP_DIFFERENCE_MODE;
     case PSP_BLEND_DODGE:
     case PSP_BLEND_BURN:
     case PSP_BLEND_EXCLUSION:
@@ -945,7 +880,7 @@ read_channel_data (FILE       *f,
 		   guchar    **pixels,
 		   guint       bytespp,
 		   guint       offset,
-		   GDrawable  *drawable,
+		   GimpDrawable  *drawable,
 		   guint32     compressed_len)
 {
   gint i, y, width = drawable->width, height = drawable->height;
@@ -1035,7 +970,7 @@ read_channel_data (FILE       *f,
       zstream.opaque = f;
       if (inflateInit (&zstream) != Z_OK)
 	{
-	  g_message ("PSP: zlib error");
+	  g_message ("zlib error");
 	  fclose (f);
 	  return -1;
 	}
@@ -1049,14 +984,14 @@ read_channel_data (FILE       *f,
       zstream.avail_out = npixels;
       if (inflate (&zstream, Z_FINISH) != Z_STREAM_END)
 	{
-	  g_message ("PSP: zlib error");
+	  g_message ("zlib error");
 	  inflateEnd (&zstream);
 	  fclose (f);
 	  return -1;
 	}
       inflateEnd (&zstream);
       g_free (buf);
-      
+
       if (bytespp > 1)
 	{
 	  p = buf2;
@@ -1084,23 +1019,23 @@ read_layer_block (FILE     *f,
   long block_start, sub_block_start, channel_start;
   gint sub_id;
   guint32 sub_init_len, sub_total_len;
-  guchar *name;
+  gchar *name;
   guint16 namelen;
   guchar type, opacity, blend_mode, visibility, transparency_protected;
   guchar link_group_id, mask_linked, mask_disabled;
   guint32 image_rect[4], saved_image_rect[4], mask_rect[4], saved_mask_rect[4];
   gboolean null_layer = FALSE;
   guint16 bitmap_count, channel_count;
-  GDrawableType drawable_type;
+  GimpImageType drawable_type;
   guint32 layer_ID = 0;
-  GLayerMode layer_mode;
+  GimpLayerModeEffects layer_mode;
   guint32 channel_init_len, channel_total_len;
   guint32 compressed_len, uncompressed_len;
   guint16 bitmap_type, channel_type;
   gint width, height, bytespp, offset;
   guchar **pixels, *pixel;
-  GDrawable *drawable;
-  GPixelRgn pixel_rgn;
+  GimpDrawable *drawable;
+  GimpPixelRgn pixel_rgn;
 
   block_start = ftell (f);
 
@@ -1115,7 +1050,7 @@ read_layer_block (FILE     *f,
 	}
       if (sub_id != PSP_LAYER_BLOCK)
 	{
-	  g_message ("PSP: Invalid layer sub-block %s, should be LAYER",
+	  g_message ("Invalid layer sub-block %s, should be LAYER",
 		     block_name (sub_id));
 	  fclose (f);
 	  gimp_image_delete (image_ID);
@@ -1148,7 +1083,7 @@ read_layer_block (FILE     *f,
 	      || fread (&bitmap_count, 2, 1, f) < 1
 	      || fread (&channel_count, 2, 1, f) < 1)
 	    {
-	      g_message ("PSP: Error reading layer information chunk");
+	      g_message ("Error reading layer information chunk");
 	      fclose (f);
 	      gimp_image_delete (image_ID);
 	      return -1;
@@ -1177,7 +1112,7 @@ read_layer_block (FILE     *f,
 	      || fread (&bitmap_count, 2, 1, f) < 1
 	      || fread (&channel_count, 2, 1, f) < 1)
 	    {
-	      g_message ("PSP: Error reading layer information chunk");
+	      g_message ("Error reading layer information chunk");
 	      g_free (name);
 	      fclose (f);
 	      gimp_image_delete (image_ID);
@@ -1186,7 +1121,7 @@ read_layer_block (FILE     *f,
 	}
 
       if (type == PSP_LAYER_FLOATING_SELECTION)
-	g_message ("PSP: Floating selection restored as normal layer");
+	g_message ("Floating selection restored as normal layer");
 
       swab_rect (image_rect);
       swab_rect (saved_image_rect);
@@ -1198,10 +1133,10 @@ read_layer_block (FILE     *f,
       layer_mode = gimp_layer_mode_from_psp_blend_mode (blend_mode);
       if ((int) layer_mode == -1)
 	{
-	  g_message ("PSP: Unsupported PSP layer blend mode %s "
+	  g_message ("Unsupported PSP layer blend mode %s "
 		     "for layer %s, setting layer invisible",
 		     blend_mode_name (blend_mode), name);
-	  layer_mode = NORMAL_MODE;
+	  layer_mode = GIMP_NORMAL_MODE;
 	  visibility = FALSE;
 	}
 
@@ -1209,7 +1144,7 @@ read_layer_block (FILE     *f,
       height = saved_image_rect[3] - saved_image_rect[1];
 
       IFDBG(2) g_message
-	("PSP: layer: %s %dx%d (%dx%d) @%d,%d opacity %d blend_mode %s "
+	("layer: %s %dx%d (%dx%d) @%d,%d opacity %d blend_mode %s "
 	 "%d bitmaps %d channels",
 	 name,
 	 image_rect[2] - image_rect[0], image_rect[3] - image_rect[1],
@@ -1219,7 +1154,7 @@ read_layer_block (FILE     *f,
 	 bitmap_count, channel_count);
 
       IFDBG(2) g_message
-	("PSP: mask %dx%d (%dx%d) @%d,%d",
+	("mask %dx%d (%dx%d) @%d,%d",
 	 mask_rect[2] - mask_rect[0],
 	 mask_rect[3] - mask_rect[1],
 	 saved_mask_rect[2] - saved_mask_rect[0],
@@ -1239,14 +1174,14 @@ read_layer_block (FILE     *f,
 
       if (ia->greyscale)
 	if (!null_layer && bitmap_count == 1)
-	  drawable_type = GRAY_IMAGE, bytespp = 1;
+	  drawable_type = GIMP_GRAY_IMAGE, bytespp = 1;
 	else
-	  drawable_type = GRAYA_IMAGE, bytespp = 1;
+	  drawable_type = GIMP_GRAYA_IMAGE, bytespp = 1;
       else
 	if (!null_layer && bitmap_count == 1)
-	  drawable_type = RGB_IMAGE, bytespp = 3;
+	  drawable_type = GIMP_RGB_IMAGE, bytespp = 3;
 	else
-	  drawable_type = RGBA_IMAGE, bytespp = 4;
+	  drawable_type = GIMP_RGBA_IMAGE, bytespp = 4;
 
       layer_ID = gimp_layer_new (image_ID, name,
 				 width, height,
@@ -1255,7 +1190,7 @@ read_layer_block (FILE     *f,
 				 layer_mode);
       if (layer_ID == -1)
 	{
-	  g_message ("PSP: Error creating layer");
+	  g_message ("Error creating layer");
 	  fclose (f);
 	  gimp_image_delete (image_ID);
 	  return -1;
@@ -1270,9 +1205,9 @@ read_layer_block (FILE     *f,
 				saved_image_rect[0], saved_image_rect[1]);
 
       if (!visibility)
-	gimp_layer_set_visible (layer_ID, FALSE);
-      
-      gimp_layer_set_preserve_transparency (layer_ID, transparency_protected);
+	gimp_drawable_set_visible (layer_ID, FALSE);
+
+      gimp_layer_set_preserve_trans (layer_ID, transparency_protected);
 
       if (major < 4)
 	if (try_fseek (f, sub_block_start + sub_init_len, SEEK_SET) < 0)
@@ -1295,7 +1230,7 @@ read_layer_block (FILE     *f,
       gimp_pixel_rgn_init (&pixel_rgn, drawable, 0, 0,
 			   width, height, TRUE, FALSE);
 
-      gimp_tile_cache_size (tile_height * width * bytespp);
+      gimp_tile_cache_size (gimp_tile_height () * width * bytespp);
 
       /* Read the layer channel sub-blocks */
       while (ftell (f) < sub_block_start + sub_total_len)
@@ -1310,7 +1245,7 @@ read_layer_block (FILE     *f,
 
 	  if (sub_id != PSP_CHANNEL_BLOCK)
 	    {
-	      g_message ("PSP: Invalid layer sub-block %s, should be CHANNEL",
+	      g_message ("Invalid layer sub-block %s, should be CHANNEL",
 			 block_name (sub_id));
 	      fclose (f);
 	      gimp_image_delete (image_ID);
@@ -1321,13 +1256,13 @@ read_layer_block (FILE     *f,
 
 	  if (major == 4)
 	    fseek (f, 4, SEEK_CUR); /* Unknown field */
-	  
+
 	  if (fread (&compressed_len, 4, 1, f) < 1
 	      || fread (&uncompressed_len, 4, 1, f) < 1
 	      || fread (&bitmap_type, 2, 1, f) < 1
 	      || fread (&channel_type, 2, 1, f) < 1)
 	    {
-	      g_message ("PSP: Error reading channel information chunk");
+	      g_message ("Error reading channel information chunk");
 	      fclose (f);
 	      gimp_image_delete (image_ID);
 	      return -1;
@@ -1340,8 +1275,7 @@ read_layer_block (FILE     *f,
 
 	  if (bitmap_type > PSP_DIB_USER_MASK)
 	    {
-	      g_message ("PSP: Invalid bitmap type %d "
-			 "in channel information chunk",
+	      g_message ("Invalid bitmap type %d in channel information chunk",
 			 bitmap_type);
 	      fclose (f);
 	      gimp_image_delete (image_ID);
@@ -1350,16 +1284,14 @@ read_layer_block (FILE     *f,
 
 	  if (channel_type > PSP_CHANNEL_BLUE)
 	    {
-	      g_message ("PSP: Invalid channel type %d "
-			 "in channel information chunk",
+	      g_message ("Invalid channel type %d in channel information chunk",
 			 channel_type);
 	      fclose (f);
 	      gimp_image_delete (image_ID);
 	      return -1;
 	    }
 
-	  IFDBG(2) g_message ("PSP: channel: %s %s %d (%d) bytes "
-			      "%d bytespp",
+	  IFDBG(2) g_message ("channel: %s %s %d (%d) bytes %d bytespp",
 			      bitmap_type_name (bitmap_type),
 			      channel_type_name (channel_type),
 			      uncompressed_len, compressed_len,
@@ -1376,7 +1308,7 @@ read_layer_block (FILE     *f,
 		gimp_image_delete (image_ID);
 		return -1;
 	      }
-	  
+
 	  if (!null_layer)
 	    if (read_channel_data (f, ia, pixels, bytespp,
 				   offset, drawable, compressed_len) == -1)
@@ -1420,11 +1352,11 @@ read_tube_block (FILE     *f,
   guint32 step_size, column_count, row_count, cell_count;
   guint32 placement_mode, selection_mode;
   gint i;
-  PixPipeParams params;
-  Parasite *pipe_parasite;
+  GimpPixPipeParams params;
+  GimpParasite *pipe_parasite;
   gchar *parasite_text;
 
-  pixpipeparams_init (&params);
+  gimp_pixpipe_params_init (&params);
 
   if (fread (&version, 2, 1, f) < 1
       || fread (name, 513, 1, f) < 1
@@ -1435,7 +1367,7 @@ read_tube_block (FILE     *f,
       || fread (&placement_mode, 4, 1, f) < 1
       || fread (&selection_mode, 4, 1, f) < 1)
     {
-      g_message ("PSP: Error reading tube data chunk");
+      g_message ("Error reading tube data chunk");
       fclose (f);
       gimp_image_delete (image_ID);
       return -1;
@@ -1471,15 +1403,15 @@ read_tube_block (FILE     *f,
 			   (selection_mode == tsmPressure ? "pressure" :
 			    (selection_mode == tsmVelocity ? "velocity" :
 			     "default")))));
-  parasite_text = pixpipeparams_build (&params);
+  parasite_text = gimp_pixpipe_params_build (&params);
 
   IFDBG(2) g_message ("parasite: %s", parasite_text);
 
-  pipe_parasite = parasite_new ("gimp-brush-pipe-parameters",
-				PARASITE_PERSISTENT,
-				strlen (parasite_text) + 1, parasite_text);
+  pipe_parasite = gimp_parasite_new ("gimp-brush-pipe-parameters",
+				     GIMP_PARASITE_PERSISTENT,
+				     strlen (parasite_text) + 1, parasite_text);
   gimp_image_parasite_attach (image_ID, pipe_parasite);
-  parasite_free (pipe_parasite);
+  gimp_parasite_free (pipe_parasite);
   g_free (parasite_text);
 
   return 0;
@@ -1503,7 +1435,7 @@ compression_name (gint compression)
 }
 
 static gint32
-load_image (gchar *filename)
+load_image (const gchar *filename)
 {
   FILE *f;
   struct stat st;
@@ -1521,20 +1453,24 @@ load_image (gchar *filename)
 
   f = fopen (filename, "rb");
   if (f == NULL)
-    return -1;
+    {
+      g_message (_("Could not open '%s' for reading: %s"),
+                 gimp_filename_to_utf8 (filename), g_strerror (errno));
+      return -1;
+    }
 
   /* Read thePSP File Header */
   if (fread (buf, 32, 1, f) < 1
       || fread (&major, 2, 1, f) < 1
       || fread (&minor, 2, 1, f) < 1)
     {
-      g_message ("PSP: Error reading file header");
+      g_message ("Error reading file header");
       fclose (f);
       return -1;
     }
   if (memcmp (buf, "Paint Shop Pro Image File\n\032\0\0\0\0\0", 32) != 0)
     {
-      g_message ("PSP: Incorrect file signature");
+      g_message ("Incorrect file signature");
       fclose (f);
       return -1;
     }
@@ -1548,7 +1484,7 @@ load_image (gchar *filename)
    */
   if (major < 3)
     {
-      g_message ("PSP: Unsupported PSP file format version "
+      g_message ("Unsupported PSP file format version "
 		 "%d.%d, only knows 3.0 (and later?)",
 		 major, minor);
       fclose (f);
@@ -1557,13 +1493,13 @@ load_image (gchar *filename)
   else if (major == 3)
     ; /* OK */
   else if (major == 4 && minor == 0)
-    g_message ("PSP: Warning: PSP file format version "
+    g_message ("Warning: PSP file format version "
 	       "4.0. Support for this format version "
 	       "is based on reverse engineering, "
 	       "as no documentation has been made available");
   else
     {
-      g_message ("PSP: Unsupported PSP file format version %d.%d",
+      g_message ("Unsupported PSP file format version %d.%d",
 		 major, minor);
       fclose (f);
       return -1;
@@ -1572,7 +1508,7 @@ load_image (gchar *filename)
   /* Read all the blocks */
   block_number = 0;
 
-  IFDBG(3) g_message ("PSP: size = %d", (int)st.st_size);
+  IFDBG(3) g_message ("size = %d", (int)st.st_size);
   while (ftell (f) != st.st_size
 	 && (id = read_block_header (f, &block_init_len,
 				     &block_total_len)) != -1)
@@ -1583,7 +1519,7 @@ load_image (gchar *filename)
 	{
 	  if (block_number != 0)
 	    {
-	      g_message ("PSP: Duplicate General Image Attributes block");
+	      g_message ("Duplicate General Image Attributes block");
 	      fclose (f);
 	      return -1;
 	    }
@@ -1591,13 +1527,13 @@ load_image (gchar *filename)
 						  block_total_len, &ia) == -1)
 	    return -1;
 
-	  IFDBG(2) g_message ("PSP: %d dpi %dx%d %s",
+	  IFDBG(2) g_message ("%d dpi %dx%d %s",
 			      (int) ia.resolution,
 			      ia.width, ia.height,
 			      compression_name (ia.compression));
 
 	  image_ID = gimp_image_new (ia.width, ia.height,
-				     ia.greyscale ? GRAY : RGB);
+				     ia.greyscale ? GIMP_GRAY : GIMP_RGB);
 	  if (image_ID == -1)
 	    return -1;
 
@@ -1609,7 +1545,7 @@ load_image (gchar *filename)
 	{
 	  if (block_number == 0)
 	    {
-	      g_message ("PSP: Missing General Image Attributes block");
+	      g_message ("Missing General Image Attributes block");
 	      fclose (f);
 	      gimp_image_delete (image_ID);
 	      return -1;
@@ -1649,13 +1585,13 @@ load_image (gchar *filename)
 	    case PSP_LAYER_BLOCK:
 	    case PSP_CHANNEL_BLOCK:
 	    case PSP_ALPHA_CHANNEL_BLOCK:
-	      g_message ("PSP: Sub-block %s should not occur "
+	      g_message ("Sub-block %s should not occur "
 			 "at main level of file",
 			 block_name (id));
 	      break;
 
 	    default:
-	      g_message ("PSP: Unrecognized block id %d", id);
+	      g_message ("Unrecognized block id %d", id);
 	      break;
 	    }
 	}
@@ -1684,37 +1620,38 @@ load_image (gchar *filename)
 }
 
 static gint
-save_image (gchar  *filename,
-	    gint32  image_ID,
-	    gint32  drawable_ID)
+save_image (const gchar *filename,
+	    gint32       image_ID,
+	    gint32       drawable_ID)
 {
-  g_message ("PSP: Saving not implemented yet");
+  g_message ("Saving not implemented yet");
 
   return FALSE;
 }
 
 static void
-run (gchar   *name,
-     gint     nparams,
-     GParam  *param,
-     gint    *nreturn_vals,
-     GParam **return_vals)
+run (const gchar      *name,
+     gint              nparams,
+     const GimpParam  *param,
+     gint             *nreturn_vals,
+     GimpParam       **return_vals)
 {
-  static GParam values[2];
-  GRunModeType  run_mode;
-  GStatusType   status = STATUS_SUCCESS;
-  gint32        image_ID;
-  gint32        drawable_ID;
-  GimpExportReturnType export = EXPORT_CANCEL;
+  static GimpParam  values[2];
+  GimpRunMode       run_mode;
+  GimpPDBStatusType status = GIMP_PDB_SUCCESS;
+  gint32            image_ID;
+  gint32            drawable_ID;
+  GimpExportReturn  export = GIMP_EXPORT_CANCEL;
 
-  tile_height = gimp_tile_height ();
+  INIT_I18N ();
 
   run_mode = param[0].data.d_int32;
 
   *nreturn_vals = 1;
   *return_vals  = values;
-  values[0].type          = PARAM_STATUS;
-  values[0].data.d_status = STATUS_EXECUTION_ERROR;
+
+  values[0].type          = GIMP_PDB_STATUS;
+  values[0].data.d_status = GIMP_PDB_EXECUTION_ERROR;
 
   if (strcmp (name, "file_psp_load") == 0)
     {
@@ -1723,12 +1660,12 @@ run (gchar   *name,
       if (image_ID != -1)
 	{
 	  *nreturn_vals = 2;
-	  values[1].type         = PARAM_IMAGE;
+	  values[1].type         = GIMP_PDB_IMAGE;
 	  values[1].data.d_image = image_ID;
 	}
       else
 	{
-	  status = STATUS_EXECUTION_ERROR;
+	  status = GIMP_PDB_EXECUTION_ERROR;
 	}
     }
   else if (strcmp (name, "file_psp_save") == 0)
@@ -1736,22 +1673,21 @@ run (gchar   *name,
       image_ID = param[1].data.d_int32;
       drawable_ID = param[2].data.d_int32;
 
-      /*  eventually export the image */ 
+      /*  eventually export the image */
       switch (run_mode)
 	{
-	case RUN_INTERACTIVE:
-	case RUN_WITH_LAST_VALS:
-	  INIT_I18N_UI();
-	  init_gtk ();
-	  export = gimp_export_image (&image_ID, &drawable_ID, "PSP", 
-				      (CAN_HANDLE_RGB |
-				       CAN_HANDLE_GRAY |
-				       CAN_HANDLE_INDEXED |
-				       CAN_HANDLE_ALPHA |
-				       CAN_HANDLE_LAYERS));
-	  if (export == EXPORT_CANCEL)
+	case GIMP_RUN_INTERACTIVE:
+	case GIMP_RUN_WITH_LAST_VALS:
+	  gimp_ui_init ("psp", FALSE);
+	  export = gimp_export_image (&image_ID, &drawable_ID, "PSP",
+				      (GIMP_EXPORT_CAN_HANDLE_RGB |
+				       GIMP_EXPORT_CAN_HANDLE_GRAY |
+				       GIMP_EXPORT_CAN_HANDLE_INDEXED |
+				       GIMP_EXPORT_CAN_HANDLE_ALPHA  |
+				       GIMP_EXPORT_CAN_HANDLE_LAYERS));
+	  if (export == GIMP_EXPORT_CANCEL)
 	    {
-	      values[0].data.d_status = STATUS_CANCEL;
+	      values[0].data.d_status = GIMP_PDB_CANCEL;
 	      return;
 	    }
 	  break;
@@ -1761,21 +1697,21 @@ run (gchar   *name,
 
       switch (run_mode)
 	{
-	case RUN_INTERACTIVE:
-	  
+	case GIMP_RUN_INTERACTIVE:
+
 	  /*  Possibly retrieve data  */
 	  gimp_get_data ("file_pnm_save", &psvals);
 
 	  /*  First acquire information with a dialog  */
 	  if (! save_dialog ())
-	    status = STATUS_CANCEL;
+	    status = GIMP_PDB_CANCEL;
 	  break;
 
-	case RUN_NONINTERACTIVE:
+	case GIMP_RUN_NONINTERACTIVE:
 	  /*  Make sure all the arguments are there!  */
 	  if (nparams != 6)
 	    {
-	      status = STATUS_CALLING_ERROR;
+	      status = GIMP_PDB_CALLING_ERROR;
 	    }
 	  else
 	    {
@@ -1783,10 +1719,10 @@ run (gchar   *name,
 
 	      if (param[5].data.d_int32 < 0 ||
 		  param[5].data.d_int32 > PSP_COMP_LZ77)
-		status = STATUS_CALLING_ERROR;
+		status = GIMP_PDB_CALLING_ERROR;
 	    }
 
-	case RUN_WITH_LAST_VALS:
+	case GIMP_RUN_WITH_LAST_VALS:
 	  gimp_get_data ("file_psp_save", &psvals);
 	  break;
 
@@ -1794,7 +1730,7 @@ run (gchar   *name,
 	  break;
 	}
 
-      if (status == STATUS_SUCCESS)
+      if (status == GIMP_PDB_SUCCESS)
 	{
 	  if (save_image (param[3].data.d_string, image_ID, drawable_ID))
 	    {
@@ -1802,16 +1738,16 @@ run (gchar   *name,
 	    }
 	  else
 	    {
-	      status = STATUS_EXECUTION_ERROR;
+	      status = GIMP_PDB_EXECUTION_ERROR;
 	    }
 	}
 
-      if (export == EXPORT_EXPORT)
+      if (export == GIMP_EXPORT_EXPORT)
 	gimp_image_delete (image_ID);
     }
   else
     {
-      status = STATUS_CALLING_ERROR;
+      status = GIMP_PDB_CALLING_ERROR;
     }
 
   values[0].data.d_status = status;
