@@ -3,7 +3,7 @@
  *
  * Generates clickable image maps.
  *
- * Copyright (C) 1998-2002 Maurits Rijk  lpeek.mrijk@consunet.nl
+ * Copyright (C) 1998-2004 Maurits Rijk  m.rijk@chello.nl
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -35,8 +35,8 @@ struct Object_t {
    ObjectClass_t 	*class;
    ObjectList_t		*list;
    gint			 refcount;
-   gint 		 selected;
-   gint			 locked;
+   gboolean 		 selected;
+   gboolean		 locked;
    gchar		*url;
    gchar 		*target;
    gchar 		*comment;
