@@ -1,5 +1,5 @@
 /*
- * "$Id: sgi.h,v 1.10 1999/06/29 06:15:15 yosh Exp $"
+ * "$Id: sgi.h,v 1.11 2003/04/07 11:59:33 neo Exp $"
  *
  *   SGI image file format library definitions.
  *
@@ -64,7 +64,8 @@ typedef struct
   FILE			*file;		/* Image file */
   int			mode,		/* File open mode */
 			bpp,		/* Bytes per pixel/channel */
-			comp;		/* Compression */
+			comp,		/* Compression */
+			swapBytes;	/* SwapBytes flag */
   unsigned short	xsize,		/* Width in pixels */
 			ysize,		/* Height in pixels */
 			zsize;		/* Number of channels */
@@ -96,5 +97,5 @@ extern int	sgiPutRow(sgi_t *sgip, unsigned short *row, int y, int z);
 #endif /* !_SGI_H_ */
 
 /*
- * End of "$Id: sgi.h,v 1.10 1999/06/29 06:15:15 yosh Exp $".
+ * End of "$Id: sgi.h,v 1.11 2003/04/07 11:59:33 neo Exp $".
  */
