@@ -1,5 +1,5 @@
 # The GIMP -- an image manipulation program
-# Copyright (C) 1998-2000 Manish Singh <yosh@gimp.org>
+# Copyright (C) 1998-2003 Manish Singh <yosh@gimp.org>
 
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -17,7 +17,7 @@
 
 package Gimp::CodeGen::app;
 
-$destdir = "$main::destdir/app";
+$destdir = "$main::destdir/app/pdb";
 
 *arg_types = \%Gimp::CodeGen::pdb::arg_types;
 *arg_parse = \&Gimp::CodeGen::pdb::arg_parse;
@@ -54,9 +54,10 @@ sub format_code_frag {
     else {
 	$code =~ s/^/' ' x ($indent ? 4 : 2)/meg;
     }
-    while ($code =~ /^\t* {8}/m) { $code =~ s/^(\t*) {8}/$1\t/mg }
     $code .= "\n";
 
+    $code =~ s/^\s+$//mg;
+
     $code;
 }
 
@@ -127,10 +128,6 @@ sub declare_args {
 	    my ($type, $name) = &arg_parse($_->{type});
 	    my $arg = $arg_types{$type};
 
-	    if ($type eq 'enum') {
-		$out->{headers}->{qq/"$enums{$name}->{header}"/}++
-	    }
-
 	    if ($arg->{array} && !exists $_->{array}) {
 		warn "Array without number of elements param in $proc->{name}";
 	    }
@@ -139,7 +136,8 @@ sub declare_args {
 		my $type = exists $_->{no_id_lookup} ? 'gint32 ' : $arg->{type};
 
 		$result .= ' ' x 2 . $type . &arg_vname($_);
-		if (!exists $_->{no_init} && exists $_->{init}) {
+		if (!exists $_->{no_init} && exists $_->{init} && 
+		    !exists $arg->{struct}) {
 		    for ($arg->{type}) {
 			/\*$/     && do { $result .= ' = NULL';  last };
 			/boolean/ && do { $result .= ' = FALSE'; last };
@@ -195,13 +193,7 @@ sub make_arg_recs {
 		    /enum/      && do { my $enum = $enums{$name};
 					$info = $enum->{info};
 					foreach (@remove) {
-					    if (exists $enum->{nicks}->{$_}) {
-						$nick = $enum->{nicks}->{$_};
-					    }
-					    else {
-						$nick = $_;
-					    }
-					    $info =~ s/$nick \(.*?\)(, )?//
+					    $info =~ s/$_ \(.*?\)(, )?//
 					}				 
 					$info =~ s/, $//;
 					if (!$#{[$info =~ /,/g]} &&
@@ -214,7 +206,7 @@ sub make_arg_recs {
 
 		$result .= <<CODE;
   {
-    PDB_$arg_types{$type}->{name},
+    GIMP_PDB_$arg_types{$type}->{name},
     "$arg->{name}",
     @{[ &quotewrap($desc, 4) ]}
   },
@@ -247,9 +239,17 @@ sub marshal_inargs {
 	    my $id_func = $arg->{id_func};
 	    $id_func = $_->{id_func} if exists $_->{id_func};
 
-	    $result .= "  $var = $id_func ($value);\n";
-	    $result .= &make_arg_test($_, sub { ${$_[0]} =~ s/==/!=/ },
-				      "$var == NULL");
+	    $result .= "  $var = $id_func (gimp, $value);\n";
+
+	    if (exists $arg->{check_func}) {
+		my $check_func = eval qq/"$arg->{check_func}"/;
+
+		$result .= &make_arg_test($_, sub { ${$_[0]} =~ s/==/!=/ },
+		                          "! $check_func");
+	    } else {
+		$result .= &make_arg_test($_, sub { ${$_[0]} =~ s/==/!=/ },
+				          "$var == NULL");
+	    }
 	}
 	else {
 	    $result .= ' ' x 2 . "$var = $value";
@@ -257,8 +257,36 @@ sub marshal_inargs {
 	    $result .= ";\n";
 
 	    if ($pdbtype eq 'string' || $pdbtype eq 'parasite') {
-		$result .= &make_arg_test($_, sub { ${$_[0]} =~ s/==/!=/ },
-					  "$var == NULL");
+		my ($reverse, $test, $utf8, $utf8testvar);
+
+		$test = "$var == NULL";
+                $utf8 = 1;
+
+		if ($pdbtype eq 'parasite') {
+		    $test .= " || $var->name == NULL";
+		    $utf8testvar = "$var->name";
+		}
+		else {
+		    $utf8 = !exists $_->{no_validate};
+		    $utf8testvar = "$var";
+		}
+
+		if (exists $_->{null_ok}) {
+		    $reverse = sub { ${$_[0]} =~ s/!//; };
+		    $test = "$var && !g_utf8_validate ($var, -1, NULL)";
+		}
+		elsif ($utf8) {
+		    $reverse = sub { ${$_[0]} =~ s/!//;
+				     ${$_[0]} =~ s/||/&&/g;
+				     ${$_[0]} =~ s/==/!=/g };
+                    $test .= " || !g_utf8_validate ($utf8testvar, -1, NULL)";
+		}
+		else {
+		    $reverse = sub { ${$_[0]} =~ s/||/&&/g;
+				     ${$_[0]} =~ s/==/!=/g };
+		}
+
+		$result .= &make_arg_test($_, $reverse, $test);
 	    }
 	    elsif ($pdbtype eq 'tattoo') {
 		$result .= &make_arg_test($_, sub { ${$_[0]} =~ s/==/!=/ },
@@ -268,7 +296,7 @@ sub marshal_inargs {
 		$typeinfo[0] = 'GIMP_UNIT_PIXEL' unless defined $typeinfo[0];
 		$result .= &make_arg_test($_, sub { ${$_[0]} = "!(${$_[0]})" },
 					  "$var < $typeinfo[0] || $var >= " .
-					  'gimp_unit_get_number_of_units ()');
+					  '_gimp_unit_get_number_of_units (gimp)');
 	    }
 	    elsif ($pdbtype eq 'enum' && !$enums{$typeinfo[0]}->{contig}) {
 		if (!exists $_->{no_success} || exists $_->{on_success} ||
@@ -293,7 +321,6 @@ sub marshal_inargs {
 			    $frag = &format_code_frag($arg->{$key}, 1);
 			    $frag =~ s/\t/' ' x 8/eg;
 			    $frag =~ s/^/' ' x 2/meg;
-			    $frag =~ s/^ {8}/\t/mg;
 			}
 			$frag;
 		    }
@@ -461,7 +488,7 @@ sub generate {
 	$out->{procs} .= "static ProcRecord ${name}_proc;\n";
 
 	$out->{register} .= <<CODE;
-  procedural_db_register (\&${name}_proc);
+  procedural_db_register (gimp, \&${name}_proc);
 CODE
 
 	if (exists $proc->{invoke}->{headers}) {
@@ -471,7 +498,10 @@ CODE
 	}
 
 	$out->{code} .= "\nstatic Argument *\n";
-	$out->{code} .= "${name}_invoker (Argument *args)\n{\n";
+	$out->{code} .= "${name}_invoker (Gimp         *gimp,\n";
+	$out->{code} .=  ' ' x length($name) . "          GimpContext  *context,\n";
+	$out->{code} .=  ' ' x length($name) . "          GimpProgress *progress,\n";
+	$out->{code} .=  ' ' x length($name) . "          Argument     *args)\n{\n";
 
 	my $code = "";
 
@@ -480,7 +510,7 @@ CODE
 	    my ($exec, $fail, $argtype);
 	    my $custom = $proc->{invoke}->{code};
 
-	    $exec = "procedural_db_execute ($procname, $args)";
+	    $exec = "procedural_db_execute (gimp, context, progress, $procname, $args)";
 	    $fail = "procedural_db_return_args (\&${name}_proc, FALSE)";
 
 	    $argtype = 'Argument';
@@ -571,7 +601,7 @@ CODE
 		    my $type = &arg_ptype($arg);
 
 		    $code .= <<CODE;
-  argv[$pos].arg_type = PDB_$arg->{name};
+  argv[$pos].arg_type = GIMP_PDB_$arg->{name};
 CODE
 
 		    my $frag = $_->{code};
@@ -584,7 +614,7 @@ CODE
 	    }
 
 	    $code .= <<CODE;
-  return $invoke->{pass_through}_invoker (argv);
+  return $invoke->{pass_through}_invoker (gimp, context, progress, argv);
 }
 CODE
 	}
@@ -634,7 +664,8 @@ static ProcRecord ${name}_proc =
   "$proc->{author}",
   "$proc->{copyright}",
   "$proc->{date}",
-  PDB_INTERNAL,
+  @{[$proc->{deprecated} ? "\"$proc->{deprecated}\"" : 'NULL']},
+  GIMP_INTERNAL,
   @{[scalar @inargs]},
   @{[scalar @inargs ? "${name}_inargs" : 'NULL']},
   @{[scalar @outargs]},
@@ -646,7 +677,7 @@ CODE
 
     my $gpl = <<'GPL';
 /* The GIMP -- an image manipulation program
- * Copyright (C) 1995-2000 Spencer Kimball and Peter Mattis
+ * Copyright (C) 1995-2003 Spencer Kimball and Peter Mattis
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -667,43 +698,101 @@ CODE
 
 GPL
 
-    my $group_procs = ""; my $longest = 0;
-    my $once = 0; my $pcount = 0.0;
+    my $group_procs = "";
+    my $longest = 0;
+    my $once = 0;
+    my $pcount = 0.0;
+
     foreach $group (@main::groups) {
 	my $out = $out{$group};
 
 	foreach (@{$main::grp{$group}->{headers}}) { $out->{headers}->{$_}++ }
 	delete $out->{headers}->{q/"procedural_db.h"/};
+	delete $out->{headers}->{q/"config.h"/};
 
 	my @headers = sort {
 	    my ($x, $y) = ($a, $b);
 	    foreach ($x, $y) {
-		$_ = "!$_" if $_ eq '"config.h"';
-		$_ = "~$_" if /libgimp/;
-		s/^</!/;
+		if (/^</) {
+		    s/^</!/;
+		}
+		elsif (!/libgimp/) {
+		    s/^/~/;
+		}
 	    }
 	    $x cmp $y;
 	} keys %{$out->{headers}};
-	my $headers = ""; my $lib = 0; my $seen = 0;
+        my $headers = ""; my $lib = 0; my $seen = 0; my $nl = 0;
+	my $sys = 0; my $base = 0;
 	foreach (@headers) {
+	    $headers .= "\n" if $nl;
+	    $nl = 0;
+
 	    if ($_ eq '<unistd.h>') {
 		$headers .= "\n" if $seen;
 		$headers .= "#ifdef HAVE_UNISTD_H\n";
 	    }
+	    if ($_ eq '<process.h>') {
+		$headers .= "\n" if $seen;
+		$headers .= "#include <glib.h>\n\n";	
+		$headers .= "#ifdef G_OS_WIN32\n";
+	    }
+
 
 	    $seen++ if /^</;
-	    $headers .= "\n" if $seen && !/^</;
+
+	    if ($sys == 0 && !/^</) {
+		$sys = 1;
+		$headers .= "\n";
+		$headers .= '#include <glib-object.h>';
+		$headers .= "\n\n";
+	    }
+
 	    $seen = 0 if !/^</;
 
-	    $headers .= "\n" if /libgimp/ && !$lib++ && $headers;
+	    if (/libgimp/) {
+		$lib = 1;
+	    }
+	    else {
+		$headers .= "\n" if $lib;
+		$lib = 0;
+
+		if ($sys == 1 && $base == 0) {
+		    $base = 1;
+
+		    $headers .= '#include "pdb-types.h"';
+		    $headers .= "\n";
+		    $headers .= '#include "procedural_db.h"';
+		    $headers .= "\n\n";
+		}
+	    }
+
+            if ($_ eq '"regexrepl/regex.h"') {
+		$headers .= "\n";
+		$headers .= "#ifdef HAVE_GLIBC_REGEX\n";
+		$headers .= "#include <regex.h>\n";
+		$headers .= "#else\n";
+	    }
 
 	    $headers .= "#include $_\n";
 
+            if ($_ eq '"regexrepl/regex.h"') {
+		$headers .= "#endif\n";
+		$nl = 1;
+	    }
+
 	    if ($_ eq '<unistd.h>') {
-		$headers .= "#endif\n\n";
+		$headers .= "#endif\n";
 		$seen = 0;
+		$nl = 1;
  	    }
 
+            if ($_ eq '<process.h>') {
+		$headers .= "#endif\n";
+		$seen = 0;
+		$nl = 1;
+	    }
+
 	    $headers .= "\n" if $_ eq '"config.h"';
 	}
 
@@ -715,11 +804,11 @@ GPL
 	my $cfile = "$destdir/${group}_cmds.c$FILE_EXT";
 	open CFILE, "> $cfile" or die "Can't open $cfile: $!\n";
 	print CFILE $gpl;
-	print CFILE qq/#include "procedural_db.h"\n\n/;
+	print CFILE qq/#include "config.h"\n\n/;
 	print CFILE $headers, "\n";
 	print CFILE $extra->{decls}, "\n" if exists $extra->{decls};
 	print CFILE $out->{procs};
-	print CFILE "\nvoid\nregister_${group}_procs (void)\n";
+	print CFILE "\nvoid\nregister_${group}_procs (Gimp *gimp)\n";
 	print CFILE "{\n$out->{register}}\n";
 	print CFILE "\n", $extra->{code} if exists $extra->{code};
 	print CFILE $out->{code};
@@ -730,26 +819,27 @@ GPL
 	push @group_decls, $decl;
 	$longest = length $decl if $longest < length $decl;
 
-	$group_procs .= ' ' x 2 . "app_init_update_status (";
+	$group_procs .= ' ' x 2 . "(* status_callback) (";
 	$group_procs .= q/_("Internal Procedures")/ unless $once;
 	$group_procs .= 'NULL' if $once++;
 	$group_procs .= qq/, _("$main::grp{$group}->{desc}"), /;
        ($group_procs .= sprintf "%.3f", $pcount / $total) =~ s/\.?0*$//s;
 	$group_procs .= ($group_procs !~ /\.\d+$/s ? ".0" : "") . ");\n";
-	$group_procs .=  ' ' x 2 . "register_${group}_procs ();\n\n";
+	$group_procs .=  ' ' x 2 . "register_${group}_procs (gimp);\n\n";
 	$pcount += $out->{pcount};
     }
 
     if (! $ENV{PDBGEN_GROUPS}) {
 	my $internal = "$destdir/internal_procs.h$FILE_EXT";
-	open IFILE, "> $internal" or die "Can't open $cmdfile: $!\n";
+	open IFILE, "> $internal" or die "Can't open $internal: $!\n";
 	print IFILE $gpl;
 	my $guard = "__INTERNAL_PROCS_H__";
 	print IFILE <<HEADER;
 #ifndef $guard
 #define $guard
 
-void internal_procs_init (void);
+void internal_procs_init (Gimp               *gimp,
+                          GimpInitStatusFunc  status_callback);
 
 #endif /* $guard */
 HEADER
@@ -757,19 +847,30 @@ HEADER
 	&write_file($internal);
 
 	$internal = "$destdir/internal_procs.c$FILE_EXT";
-	open IFILE, "> $internal" or die "Can't open $cmdfile: $!\n";
+	open IFILE, "> $internal" or die "Can't open $internal: $!\n";
 	print IFILE $gpl;
 	print IFILE qq@#include "config.h"\n\n@;
-	print IFILE qq@#include <glib.h>\n@;
-	print IFILE qq@#include "app_procs.h"\n\n@;
-	print IFILE qq@#include "libgimp/gimpintl.h"\n\n@;
+	print IFILE qq@#include <glib-object.h>\n\n@;
+	print IFILE qq@#include "pdb-types.h"\n\n@;
+	print IFILE qq@#include "core/gimp.h"\n\n@;
+	print IFILE qq@#include "gimp-intl.h"\n\n@;
 	print IFILE "/* Forward declarations for registering PDB procs */\n\n";
 	foreach (@group_decls) {
-	    print IFILE "void $_" . ' ' x ($longest - length $_) . " (void);\n";
+	    print IFILE "void $_" . ' ' x ($longest - length $_) . " (Gimp *gimp);\n";
 	}
 	chop $group_procs;
 	print IFILE "\n/* $total procedures registered total */\n\n";
-	print IFILE "void\ninternal_procs_init (void)\n{\n$group_procs}\n";
+	print IFILE <<BODY;
+void
+internal_procs_init (Gimp               *gimp,
+                     GimpInitStatusFunc  status_callback)
+{
+  g_return_if_fail (GIMP_IS_GIMP (gimp));
+  g_return_if_fail (status_callback != NULL);
+
+$group_procs
+}
+BODY
 	close IFILE;
 	&write_file($internal);
     }
